"""
Python module generated from Java source file org.bukkit.material.Redstone

Java source file obtained from artifact spigot-api version 1.21.4-R0.1-20250303.102353-42

Because this Python module is automatically generated, it may contain errors
and/or code that cannot be parsed. Please report these issues at
https://github.com/magicmq/docs-translator/issues
"""
from org.bukkit.material import *
from typing import Any, Callable, Iterable, Tuple


class Redstone:
    """
    Indicated a Material that may carry or create a Redstone current
    """

    def isPowered(self) -> bool:
        """
        Gets the current state of this Material, indicating if it's powered or
        unpowered

        Returns
        - True if powered, otherwise False
        """
        ...



"""
Python module generated from Java source file org.bukkit.plugin.PluginLoadOrder

Java source file obtained from artifact spigot-api version 1.21.4-R0.1-20250303.102353-42

Because this Python module is automatically generated, it may contain errors
and/or code that cannot be parsed. Please report these issues at
https://github.com/magicmq/docs-translator/issues
"""
from enum import Enum
from org.bukkit.plugin import *
from typing import Any, Callable, Iterable, Tuple


class PluginLoadOrder(Enum):
    """
    Represents the order in which a plugin should be initialized and enabled
    """

    STARTUP = 0
    """
    Indicates that the plugin will be loaded at startup
    """
    POSTWORLD = 1
    """
    Indicates that the plugin will be loaded after the first/default world
    was created
    """



"""
Python module generated from Java source file org.yaml.snakeyaml.tokens.TagTuple

Java source file obtained from artifact snakeyaml version 2.2

Because this Python module is automatically generated, it may contain errors
and/or code that cannot be parsed. Please report these issues at
https://github.com/magicmq/docs-translator/issues
"""
from org.yaml.snakeyaml.tokens import *
from typing import Any, Callable, Iterable, Tuple


class TagTuple:

    def __init__(self, handle: str, suffix: str):
        ...


    def getHandle(self) -> str:
        ...


    def getSuffix(self) -> str:
        ...



"""
Python module generated from Java source file com.google.common.cache.ReferenceEntry

Java source file obtained from artifact guava version 33.3.1-jre

Because this Python module is automatically generated, it may contain errors
and/or code that cannot be parsed. Please report these issues at
https://github.com/magicmq/docs-translator/issues
"""
from com.google.common.annotations import GwtIncompatible
from com.google.common.cache import *
from com.google.common.cache.LocalCache import ValueReference
from javax.annotation import CheckForNull
from typing import Any, Callable, Iterable, Tuple


class ReferenceEntry:
    """
    An entry in a reference map.
    
    Entries in the map can be in the following states:
    
    Valid:
    
    
      - Live: valid key/value are set
      - Loading: loading is pending
    
    
    Invalid:
    
    
      - Expired: time expired (key/value may still be set)
      - Collected: key/value was partially collected, but not yet cleaned up
      - Unset: marked as unset, awaiting cleanup or reuse
    """

    def getValueReference(self) -> "ValueReference"["K", "V"]:
        """
        Returns the value reference from this entry.
        """
        ...


    def setValueReference(self, valueReference: "ValueReference"["K", "V"]) -> None:
        """
        Sets the value reference for this entry.
        """
        ...


    def getNext(self) -> "ReferenceEntry"["K", "V"]:
        """
        Returns the next entry in the chain.
        """
        ...


    def getHash(self) -> int:
        """
        Returns the entry's hash.
        """
        ...


    def getKey(self) -> "K":
        """
        Returns the key for this entry.
        """
        ...


    def getAccessTime(self) -> int:
        """
        Returns the time that this entry was last accessed, in ns.
        """
        ...


    def setAccessTime(self, time: int) -> None:
        """
        Sets the entry access time in ns.
        """
        ...


    def getNextInAccessQueue(self) -> "ReferenceEntry"["K", "V"]:
        """
        Returns the next entry in the access queue.
        """
        ...


    def setNextInAccessQueue(self, next: "ReferenceEntry"["K", "V"]) -> None:
        """
        Sets the next entry in the access queue.
        """
        ...


    def getPreviousInAccessQueue(self) -> "ReferenceEntry"["K", "V"]:
        """
        Returns the previous entry in the access queue.
        """
        ...


    def setPreviousInAccessQueue(self, previous: "ReferenceEntry"["K", "V"]) -> None:
        """
        Sets the previous entry in the access queue.
        """
        ...


    def getWriteTime(self) -> int:
        ...


    def setWriteTime(self, time: int) -> None:
        """
        Sets the entry write time in ns.
        """
        ...


    def getNextInWriteQueue(self) -> "ReferenceEntry"["K", "V"]:
        """
        Returns the next entry in the write queue.
        """
        ...


    def setNextInWriteQueue(self, next: "ReferenceEntry"["K", "V"]) -> None:
        """
        Sets the next entry in the write queue.
        """
        ...


    def getPreviousInWriteQueue(self) -> "ReferenceEntry"["K", "V"]:
        """
        Returns the previous entry in the write queue.
        """
        ...


    def setPreviousInWriteQueue(self, previous: "ReferenceEntry"["K", "V"]) -> None:
        """
        Sets the previous entry in the write queue.
        """
        ...



"""
Python module generated from Java source file org.yaml.snakeyaml.introspector.PropertyUtils

Java source file obtained from artifact snakeyaml version 2.2

Because this Python module is automatically generated, it may contain errors
and/or code that cannot be parsed. Please report these issues at
https://github.com/magicmq/docs-translator/issues
"""
from java.lang.reflect import Field
from java.lang.reflect import Method
from java.lang.reflect import Modifier
from org.yaml.snakeyaml.error import YAMLException
from org.yaml.snakeyaml.introspector import *
from org.yaml.snakeyaml.util import PlatformFeatureDetector
from typing import Any, Callable, Iterable, Tuple


class PropertyUtils:

    def __init__(self):
        ...


    def getProperties(self, type: type["Object"]) -> set["Property"]:
        ...


    def getProperties(self, type: type["Object"], bAccess: "BeanAccess") -> set["Property"]:
        ...


    def getProperty(self, type: type["Object"], name: str) -> "Property":
        ...


    def getProperty(self, type: type["Object"], name: str, bAccess: "BeanAccess") -> "Property":
        ...


    def setBeanAccess(self, beanAccess: "BeanAccess") -> None:
        ...


    def setAllowReadOnlyProperties(self, allowReadOnlyProperties: bool) -> None:
        ...


    def isAllowReadOnlyProperties(self) -> bool:
        ...


    def setSkipMissingProperties(self, skipMissingProperties: bool) -> None:
        """
        Skip properties that are missing during deserialization of YAML to a Java object. The default
        is False.

        Arguments
        - skipMissingProperties: True if missing properties should be skipped, False otherwise.
        """
        ...


    def isSkipMissingProperties(self) -> bool:
        ...



"""
Python module generated from Java source file com.google.common.io.ByteSink

Java source file obtained from artifact guava version 33.3.1-jre

Because this Python module is automatically generated, it may contain errors
and/or code that cannot be parsed. Please report these issues at
https://github.com/magicmq/docs-translator/issues
"""
from com.google.common.annotations import GwtIncompatible
from com.google.common.annotations import J2ktIncompatible
from com.google.common.io import *
from com.google.errorprone.annotations import CanIgnoreReturnValue
from java.io import BufferedOutputStream
from java.io import IOException
from java.io import InputStream
from java.io import OutputStream
from java.io import OutputStreamWriter
from java.io import Writer
from java.nio.charset import Charset
from typing import Any, Callable, Iterable, Tuple


class ByteSink:
    """
    A destination to which bytes can be written, such as a file. Unlike an OutputStream, a
    `ByteSink` is not an open, stateful stream that can be written to and closed. Instead, it
    is an immutable *supplier* of `OutputStream` instances.
    
    `ByteSink` provides two kinds of methods:
    
    
      - **Methods that return a stream:** These methods should return a *new*, independent
          instance each time they are called. The caller is responsible for ensuring that the
          returned stream is closed.
      - **Convenience methods:** These are implementations of common operations that are
          typically implemented by opening a stream using one of the methods in the first category,
          doing something and finally closing the stream or channel that was opened.

    Author(s)
    - Colin Decker

    Since
    - 14.0
    """

    def asCharSink(self, charset: "Charset") -> "CharSink":
        """
        Returns a CharSink view of this `ByteSink` that writes characters to this sink as
        bytes encoded with the given Charset charset.
        """
        ...


    def openStream(self) -> "OutputStream":
        """
        Opens a new OutputStream for writing to this sink. This method returns a new,
        independent stream each time it is called.
        
        The caller is responsible for ensuring that the returned stream is closed.

        Raises
        - IOException: if an I/O error occurs while opening the stream
        """
        ...


    def openBufferedStream(self) -> "OutputStream":
        """
        Opens a new buffered OutputStream for writing to this sink. The returned stream is not
        required to be a BufferedOutputStream in order to allow implementations to simply
        delegate to .openStream() when the stream returned by that method does not benefit from
        additional buffering (for example, a `ByteArrayOutputStream`). This method returns a new,
        independent stream each time it is called.
        
        The caller is responsible for ensuring that the returned stream is closed.

        Raises
        - IOException: if an I/O error occurs while opening the stream

        Since
        - 15.0 (in 14.0 with return type BufferedOutputStream)
        """
        ...


    def write(self, bytes: list[int]) -> None:
        """
        Writes all the given bytes to this sink.

        Raises
        - IOException: if an I/O occurs while writing to this sink
        """
        ...


    def writeFrom(self, input: "InputStream") -> int:
        """
        Writes all the bytes from the given `InputStream` to this sink. Does not close `input`.

        Returns
        - the number of bytes written

        Raises
        - IOException: if an I/O occurs while reading from `input` or writing to this sink
        """
        ...



"""
Python module generated from Java source file org.bukkit.permissions.Permission

Java source file obtained from artifact spigot-api version 1.21.4-R0.1-20250303.102353-42

Because this Python module is automatically generated, it may contain errors
and/or code that cannot be parsed. Please report these issues at
https://github.com/magicmq/docs-translator/issues
"""
from com.google.common.base import Preconditions
from java.util import Locale
from org.bukkit import Bukkit
from org.bukkit.permissions import *
from org.bukkit.plugin import PluginManager
from typing import Any, Callable, Iterable, Tuple


class Permission:
    """
    Represents a unique permission that may be attached to a Permissible
    """

    DEFAULT_PERMISSION = PermissionDefault.OP


    def __init__(self, name: str):
        ...


    def __init__(self, name: str, description: str):
        ...


    def __init__(self, name: str, defaultValue: "PermissionDefault"):
        ...


    def __init__(self, name: str, description: str, defaultValue: "PermissionDefault"):
        ...


    def __init__(self, name: str, children: dict[str, "Boolean"]):
        ...


    def __init__(self, name: str, description: str, children: dict[str, "Boolean"]):
        ...


    def __init__(self, name: str, defaultValue: "PermissionDefault", children: dict[str, "Boolean"]):
        ...


    def __init__(self, name: str, description: str, defaultValue: "PermissionDefault", children: dict[str, "Boolean"]):
        ...


    def getName(self) -> str:
        """
        Returns the unique fully qualified name of this Permission

        Returns
        - Fully qualified name
        """
        ...


    def getChildren(self) -> dict[str, "Boolean"]:
        """
        Gets the children of this permission.
        
        If you change this map in any form, you must call .recalculatePermissibles() to recalculate all Permissibles

        Returns
        - Permission children
        """
        ...


    def getDefault(self) -> "PermissionDefault":
        """
        Gets the default value of this permission.

        Returns
        - Default value of this permission.
        """
        ...


    def setDefault(self, value: "PermissionDefault") -> None:
        """
        Sets the default value of this permission.
        
        This will not be saved to disk, and is a temporary operation until the
        server reloads permissions. Changing this default will cause all Permissibles that contain this permission to recalculate their
        permissions

        Arguments
        - value: The new default to set
        """
        ...


    def getDescription(self) -> str:
        """
        Gets a brief description of this permission, may be empty

        Returns
        - Brief description of this permission
        """
        ...


    def setDescription(self, value: str) -> None:
        """
        Sets the description of this permission.
        
        This will not be saved to disk, and is a temporary operation until the
        server reloads permissions.

        Arguments
        - value: The new description to set
        """
        ...


    def getPermissibles(self) -> set["Permissible"]:
        """
        Gets a set containing every Permissible that has this
        permission.
        
        This set cannot be modified.

        Returns
        - Set containing permissibles with this permission
        """
        ...


    def recalculatePermissibles(self) -> None:
        """
        Recalculates all Permissibles that contain this permission.
        
        This should be called after modifying the children, and is
        automatically called after modifying the default value
        """
        ...


    def addParent(self, name: str, value: bool) -> "Permission":
        """
        Adds this permission to the specified parent permission.
        
        If the parent permission does not exist, it will be created and
        registered.

        Arguments
        - name: Name of the parent permission
        - value: The value to set this permission to

        Returns
        - Parent permission it created or loaded
        """
        ...


    def addParent(self, perm: "Permission", value: bool) -> None:
        """
        Adds this permission to the specified parent permission.

        Arguments
        - perm: Parent permission to register with
        - value: The value to set this permission to
        """
        ...


    @staticmethod
    def loadPermissions(data: dict[Any, Any], error: str, def: "PermissionDefault") -> list["Permission"]:
        """
        Loads a list of Permissions from a map of data, usually used from
        retrieval from a yaml file.
        
        The data may contain a list of name:data, where the data contains the
        following keys:
        
        - default: Boolean True or False. If not specified, False.
        - children: `Map<String, Boolean>` of child permissions. If not
            specified, empty list.
        - description: Short string containing a very small description of
            this description. If not specified, empty string.

        Arguments
        - data: Map of permissions
        - error: An error message to show if a permission is invalid. May contain "%s" format tag, which will be replaced with the name of invalid permission.
        - def: Default permission value to use if missing

        Returns
        - Permission object
        """
        ...


    @staticmethod
    def loadPermission(name: str, data: dict[str, "Object"]) -> "Permission":
        """
        Loads a Permission from a map of data, usually used from retrieval from
        a yaml file.
        
        The data may contain the following keys:
        
        - default: Boolean True or False. If not specified, False.
        - children: `Map<String, Boolean>` of child permissions. If not
            specified, empty list.
        - description: Short string containing a very small description of
            this description. If not specified, empty string.

        Arguments
        - name: Name of the permission
        - data: Map of keys

        Returns
        - Permission object
        """
        ...


    @staticmethod
    def loadPermission(name: str, data: dict[Any, Any], def: "PermissionDefault", output: list["Permission"]) -> "Permission":
        """
        Loads a Permission from a map of data, usually used from retrieval from
        a yaml file.
        
        The data may contain the following keys:
        
        - default: Boolean True or False. If not specified, False.
        - children: `Map<String, Boolean>` of child permissions. If not
            specified, empty list.
        - description: Short string containing a very small description of
            this description. If not specified, empty string.

        Arguments
        - name: Name of the permission
        - data: Map of keys
        - def: Default permission value to use if not set
        - output: A list to append any created child-Permissions to, may be null

        Returns
        - Permission object
        """
        ...



"""
Python module generated from Java source file org.bukkit.block.data.type.TurtleEgg

Java source file obtained from artifact spigot-api version 1.21.4-R0.1-20250303.102353-42

Because this Python module is automatically generated, it may contain errors
and/or code that cannot be parsed. Please report these issues at
https://github.com/magicmq/docs-translator/issues
"""
from org.bukkit.block.data import Hatchable
from org.bukkit.block.data.type import *
from typing import Any, Callable, Iterable, Tuple


class TurtleEgg(Hatchable):
    """
    'eggs' is the number of eggs which appear in this block.
    """

    def getEggs(self) -> int:
        """
        Gets the value of the 'eggs' property.

        Returns
        - the 'eggs' value
        """
        ...


    def setEggs(self, eggs: int) -> None:
        """
        Sets the value of the 'eggs' property.

        Arguments
        - eggs: the new 'eggs' value
        """
        ...


    def getMinimumEggs(self) -> int:
        """
        Gets the minimum allowed value of the 'eggs' property.

        Returns
        - the minimum 'eggs' value
        """
        ...


    def getMaximumEggs(self) -> int:
        """
        Gets the maximum allowed value of the 'eggs' property.

        Returns
        - the maximum 'eggs' value
        """
        ...



"""
Python module generated from Java source file com.google.common.net.InetAddresses

Java source file obtained from artifact guava version 33.3.1-jre

Because this Python module is automatically generated, it may contain errors
and/or code that cannot be parsed. Please report these issues at
https://github.com/magicmq/docs-translator/issues
"""
from com.google.common.annotations import GwtIncompatible
from com.google.common.annotations import J2ktIncompatible
from com.google.common.base import CharMatcher
from com.google.common.base import MoreObjects
from com.google.common.hash import Hashing
from com.google.common.io import ByteStreams
from com.google.common.net import *
from com.google.common.primitives import Ints
from com.google.errorprone.annotations import CanIgnoreReturnValue
from java.math import BigInteger
from java.net import Inet4Address
from java.net import Inet6Address
from java.net import InetAddress
from java.net import NetworkInterface
from java.net import SocketException
from java.net import UnknownHostException
from java.util import Arrays
from java.util import Locale
from javax.annotation import CheckForNull
from org.checkerframework.checker.nullness.qual import Nullable
from typing import Any, Callable, Iterable, Tuple


class InetAddresses:
    """
    Static utility methods pertaining to InetAddress instances.
    
    **Important note:** Unlike `InetAddress.getByName()`, the methods of this class never
    cause DNS services to be accessed. For this reason, you should prefer these methods as much as
    possible over their JDK equivalents whenever you are expecting to handle only IP address string
    literals -- there is no blocking DNS penalty for a malformed string.
    
    When dealing with Inet4Address and Inet6Address objects as byte arrays (vis.
    `InetAddress.getAddress()`) they are 4 and 16 bytes in length, respectively, and represent
    the address in network byte order.
    
    Examples of IP addresses and their byte representations:
    
    <dl>
      <dt>The IPv4 loopback address, `"127.0.0.1"`.
      <dd>`7f 00 00 01`
      <dt>The IPv6 loopback address, `"::1"`.
      <dd>`00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 01`
      <dt>From the IPv6 reserved documentation prefix (`2001:db8::/32`), `"2001:db8::1"`.
      <dd>`20 01 0d b8 00 00 00 00 00 00 00 00 00 00 00 01`
      <dt>An IPv6 "IPv4 compatible" (or "compat") address, `"::192.168.0.1"`.
      <dd>`00 00 00 00 00 00 00 00 00 00 00 00 c0 a8 00 01`
      <dt>An IPv6 "IPv4 mapped" address, `"::ffff:192.168.0.1"`.
      <dd>`00 00 00 00 00 00 00 00 00 00 ff ff c0 a8 00 01`
    </dl>
    
    A few notes about IPv6 "IPv4 mapped" addresses and their observed use in Java.
    
    "IPv4 mapped" addresses were originally a representation of IPv4 addresses for use on an IPv6
    socket that could receive both IPv4 and IPv6 connections (by disabling the `IPV6_V6ONLY`
    socket option on an IPv6 socket). Yes, it's confusing. Nevertheless, these "mapped" addresses
    were never supposed to be seen on the wire. That assumption was dropped, some say mistakenly, in
    later RFCs with the apparent aim of making IPv4-to-IPv6 transition simpler.
    
    Technically one *can* create a 128bit IPv6 address with the wire format of a "mapped"
    address, as shown above, and transmit it in an IPv6 packet header. However, Java's InetAddress
    creation methods appear to adhere doggedly to the original intent of the "mapped" address: all
    "mapped" addresses return Inet4Address objects.
    
    For added safety, it is common for IPv6 network operators to filter all packets where either
    the source or destination address appears to be a "compat" or "mapped" address. Filtering
    suggestions usually recommend discarding any packets with source or destination addresses in the
    invalid range `::/3`, which includes both of these bizarre address formats. For more
    information on "bogons", including lists of IPv6 bogon space, see:
    
    
      - <a target="_parent"
          href="http://en.wikipedia.org/wiki/Bogon_filtering">http://en.wikipedia.
          org/wiki/Bogon_filtering</a>
      - <a target="_parent"
          href="http://www.cymru.com/Bogons/ipv6.txt">http://www.cymru.com/Bogons/ ipv6.txt</a>
      - <a target="_parent" href="http://www.cymru.com/Bogons/v6bogon.html">http://www.cymru.com/
          Bogons/v6bogon.html</a>
      - <a target="_parent" href="http://www.space.net/~gert/RIPE/ipv6-filters.html">http://www.
          space.net/~gert/RIPE/ipv6-filters.html</a>

    Author(s)
    - Erik Kline

    Since
    - 5.0
    """

    @staticmethod
    def forString(ipString: str) -> "InetAddress":
        """
        Returns the InetAddress having the given string representation.
        
        This deliberately avoids all nameservice lookups (e.g. no DNS).
        
        This method accepts non-ASCII digits, for example `"１９２.１６８.０.１"` (those are fullwidth
        characters). That is consistent with InetAddress, but not with various RFCs. If you
        want to accept ASCII digits only, you can use something like `CharMatcher.ascii().matchesAllOf(ipString)`.
        
        The scope ID is validated against the interfaces on the machine, which requires permissions
        under Android.
        
        **Android users on API >= 29:** Prefer `InetAddresses.parseNumericAddress`.

        Arguments
        - ipString: `String` containing an IPv4 or IPv6 string literal, e.g. `"192.168.0.1"` or `"2001:db8::1"` or with a scope ID, e.g. `"2001:db8::1%eth0"`

        Returns
        - InetAddress representing the argument

        Raises
        - IllegalArgumentException: if the argument is not a valid IP string literal
        """
        ...


    @staticmethod
    def isInetAddress(ipString: str) -> bool:
        """
        Returns `True` if the supplied string is a valid IP string literal, `False`
        otherwise.
        
        This method accepts non-ASCII digits, for example `"１９２.１６８.０.１"` (those are fullwidth
        characters). That is consistent with InetAddress, but not with various RFCs. If you
        want to accept ASCII digits only, you can use something like `CharMatcher.ascii().matchesAllOf(ipString)`.

        Arguments
        - ipString: `String` to evaluated as an IP string literal

        Returns
        - `True` if the argument is a valid IP string literal
        """
        ...


    @staticmethod
    def toAddrString(ip: "InetAddress") -> str:
        """
        Returns the string representation of an InetAddress.
        
        For IPv4 addresses, this is identical to InetAddress.getHostAddress(), but for IPv6
        addresses, the output follows <a href="http://tools.ietf.org/html/rfc5952">RFC 5952</a> section
        4. The main difference is that this method uses "::" for zero compression, while Java's version
        uses the uncompressed form (except on Android, where the zero compression is also done). The
        other difference is that this method outputs any scope ID in the format that it was provided at
        creation time, while Android may always output it as an interface name, even if it was supplied
        as a numeric ID.
        
        This method uses hexadecimal for all IPv6 addresses, including IPv4-mapped IPv6 addresses
        such as "::c000:201".

        Arguments
        - ip: InetAddress to be converted to an address string

        Returns
        - `String` containing the text-formatted IP address

        Since
        - 10.0
        """
        ...


    @staticmethod
    def toUriString(ip: "InetAddress") -> str:
        """
        Returns the string representation of an InetAddress suitable for inclusion in a URI.
        
        For IPv4 addresses, this is identical to InetAddress.getHostAddress(), but for IPv6
        addresses it compresses zeroes and surrounds the text with square brackets; for example `"[2001:db8::1]"`.
        
        Per section 3.2.2 of <a target="_parent"
        href="http://tools.ietf.org/html/rfc3986#section-3.2.2">RFC 3986</a>, a URI containing an IPv6
        string literal is of the form `"http://[2001:db8::1]:8888/index.html"`.
        
        Use of either InetAddresses.toAddrString, InetAddress.getHostAddress(), or
        this method is recommended over InetAddress.toString() when an IP address string
        literal is desired. This is because InetAddress.toString() prints the hostname and the
        IP address string joined by a "/".

        Arguments
        - ip: InetAddress to be converted to URI string literal

        Returns
        - `String` containing URI-safe string literal
        """
        ...


    @staticmethod
    def forUriString(hostAddr: str) -> "InetAddress":
        """
        Returns an InetAddress representing the literal IPv4 or IPv6 host portion of a URL, encoded in
        the format specified by RFC 3986 section 3.2.2.
        
        This method is similar to InetAddresses.forString(String), however, it requires that
        IPv6 addresses are surrounded by square brackets.
        
        This method is the inverse of InetAddresses.toUriString(java.net.InetAddress).
        
        This method accepts non-ASCII digits, for example `"１９２.１６８.０.１"` (those are fullwidth
        characters). That is consistent with InetAddress, but not with various RFCs. If you
        want to accept ASCII digits only, you can use something like `CharMatcher.ascii().matchesAllOf(ipString)`.

        Arguments
        - hostAddr: an RFC 3986 section 3.2.2 encoded IPv4 or IPv6 address

        Returns
        - an InetAddress representing the address in `hostAddr`

        Raises
        - IllegalArgumentException: if `hostAddr` is not a valid IPv4 address, or IPv6
            address surrounded by square brackets, or if the address has a scope id that fails
            validation against interfaces on the machine
        """
        ...


    @staticmethod
    def isUriInetAddress(ipString: str) -> bool:
        """
        Returns `True` if the supplied string is a valid URI IP string literal, `False`
        otherwise.
        
        This method accepts non-ASCII digits, for example `"１９２.１６８.０.１"` (those are fullwidth
        characters). That is consistent with InetAddress, but not with various RFCs. If you
        want to accept ASCII digits only, you can use something like `CharMatcher.ascii().matchesAllOf(ipString)`.
        
        Note that if this method returns `True`, a call to .forUriString(String) can
        throw if the address has a scope id fails validation against interfaces on the machine.

        Arguments
        - ipString: `String` to evaluated as an IP URI host string literal

        Returns
        - `True` if the argument is a valid IP URI host
        """
        ...


    @staticmethod
    def isCompatIPv4Address(ip: "Inet6Address") -> bool:
        """
        Evaluates whether the argument is an IPv6 "compat" address.
        
        An "IPv4 compatible", or "compat", address is one with 96 leading bits of zero, with the
        remaining 32 bits interpreted as an IPv4 address. These are conventionally represented in
        string literals as `"::192.168.0.1"`, though `"::c0a8:1"` is also considered an
        IPv4 compatible address (and equivalent to `"::192.168.0.1"`).
        
        For more on IPv4 compatible addresses see section 2.5.5.1 of <a target="_parent"
        href="http://tools.ietf.org/html/rfc4291#section-2.5.5.1">RFC 4291</a>.
        
        NOTE: This method is different from Inet6Address.isIPv4CompatibleAddress in that it
        more correctly classifies `"::"` and `"::1"` as proper IPv6 addresses (which they
        are), NOT IPv4 compatible addresses (which they are generally NOT considered to be).

        Arguments
        - ip: Inet6Address to be examined for embedded IPv4 compatible address format

        Returns
        - `True` if the argument is a valid "compat" address
        """
        ...


    @staticmethod
    def getCompatIPv4Address(ip: "Inet6Address") -> "Inet4Address":
        """
        Returns the IPv4 address embedded in an IPv4 compatible address.

        Arguments
        - ip: Inet6Address to be examined for an embedded IPv4 address

        Returns
        - Inet4Address of the embedded IPv4 address

        Raises
        - IllegalArgumentException: if the argument is not a valid IPv4 compatible address
        """
        ...


    @staticmethod
    def is6to4Address(ip: "Inet6Address") -> bool:
        """
        Evaluates whether the argument is a 6to4 address.
        
        6to4 addresses begin with the `"2002::/16"` prefix. The next 32 bits are the IPv4
        address of the host to which IPv6-in-IPv4 tunneled packets should be routed.
        
        For more on 6to4 addresses see section 2 of <a target="_parent"
        href="http://tools.ietf.org/html/rfc3056#section-2">RFC 3056</a>.

        Arguments
        - ip: Inet6Address to be examined for 6to4 address format

        Returns
        - `True` if the argument is a 6to4 address
        """
        ...


    @staticmethod
    def get6to4IPv4Address(ip: "Inet6Address") -> "Inet4Address":
        """
        Returns the IPv4 address embedded in a 6to4 address.

        Arguments
        - ip: Inet6Address to be examined for embedded IPv4 in 6to4 address

        Returns
        - Inet4Address of embedded IPv4 in 6to4 address

        Raises
        - IllegalArgumentException: if the argument is not a valid IPv6 6to4 address
        """
        ...


    @staticmethod
    def isTeredoAddress(ip: "Inet6Address") -> bool:
        """
        Evaluates whether the argument is a Teredo address.
        
        Teredo addresses begin with the `"2001::/32"` prefix.

        Arguments
        - ip: Inet6Address to be examined for Teredo address format

        Returns
        - `True` if the argument is a Teredo address
        """
        ...


    @staticmethod
    def getTeredoInfo(ip: "Inet6Address") -> "TeredoInfo":
        """
        Returns the Teredo information embedded in a Teredo address.

        Arguments
        - ip: Inet6Address to be examined for embedded Teredo information

        Returns
        - extracted `TeredoInfo`

        Raises
        - IllegalArgumentException: if the argument is not a valid IPv6 Teredo address
        """
        ...


    @staticmethod
    def isIsatapAddress(ip: "Inet6Address") -> bool:
        """
        Evaluates whether the argument is an ISATAP address.
        
        From RFC 5214: "ISATAP interface identifiers are constructed in Modified EUI-64 format [...]
        by concatenating the 24-bit IANA OUI (00-00-5E), the 8-bit hexadecimal value 0xFE, and a 32-bit
        IPv4 address in network byte order [...]"
        
        For more on ISATAP addresses see section 6.1 of <a target="_parent"
        href="http://tools.ietf.org/html/rfc5214#section-6.1">RFC 5214</a>.

        Arguments
        - ip: Inet6Address to be examined for ISATAP address format

        Returns
        - `True` if the argument is an ISATAP address
        """
        ...


    @staticmethod
    def getIsatapIPv4Address(ip: "Inet6Address") -> "Inet4Address":
        """
        Returns the IPv4 address embedded in an ISATAP address.

        Arguments
        - ip: Inet6Address to be examined for embedded IPv4 in ISATAP address

        Returns
        - Inet4Address of embedded IPv4 in an ISATAP address

        Raises
        - IllegalArgumentException: if the argument is not a valid IPv6 ISATAP address
        """
        ...


    @staticmethod
    def hasEmbeddedIPv4ClientAddress(ip: "Inet6Address") -> bool:
        """
        Examines the Inet6Address to determine if it is an IPv6 address of one of the specified address
        types that contain an embedded IPv4 address.
        
        NOTE: ISATAP addresses are explicitly excluded from this method due to their trivial
        spoofability. With other transition addresses spoofing involves (at least) infection of one's
        BGP routing table.

        Arguments
        - ip: Inet6Address to be examined for embedded IPv4 client address

        Returns
        - `True` if there is an embedded IPv4 client address

        Since
        - 7.0
        """
        ...


    @staticmethod
    def getEmbeddedIPv4ClientAddress(ip: "Inet6Address") -> "Inet4Address":
        """
        Examines the Inet6Address to extract the embedded IPv4 client address if the InetAddress is an
        IPv6 address of one of the specified address types that contain an embedded IPv4 address.
        
        NOTE: ISATAP addresses are explicitly excluded from this method due to their trivial
        spoofability. With other transition addresses spoofing involves (at least) infection of one's
        BGP routing table.

        Arguments
        - ip: Inet6Address to be examined for embedded IPv4 client address

        Returns
        - Inet4Address of embedded IPv4 client address

        Raises
        - IllegalArgumentException: if the argument does not have a valid embedded IPv4 address
        """
        ...


    @staticmethod
    def isMappedIPv4Address(ipString: str) -> bool:
        """
        Evaluates whether the argument is an "IPv4 mapped" IPv6 address.
        
        An "IPv4 mapped" address is anything in the range ::ffff:0:0/96 (sometimes written as
        ::ffff:0.0.0.0/96), with the last 32 bits interpreted as an IPv4 address.
        
        For more on IPv4 mapped addresses see section 2.5.5.2 of <a target="_parent"
        href="http://tools.ietf.org/html/rfc4291#section-2.5.5.2">RFC 4291</a>.
        
        Note: This method takes a `String` argument because InetAddress automatically
        collapses mapped addresses to IPv4. (It is actually possible to avoid this using one of the
        obscure Inet6Address methods, but it would be unwise to depend on such a
        poorly-documented feature.)
        
        This method accepts non-ASCII digits. That is consistent with InetAddress, but not
        with various RFCs. If you want to accept ASCII digits only, you can use something like `CharMatcher.ascii().matchesAllOf(ipString)`.

        Arguments
        - ipString: `String` to be examined for embedded IPv4-mapped IPv6 address format

        Returns
        - `True` if the argument is a valid "mapped" address

        Since
        - 10.0
        """
        ...


    @staticmethod
    def getCoercedIPv4Address(ip: "InetAddress") -> "Inet4Address":
        """
        Coerces an IPv6 address into an IPv4 address.
        
        HACK: As long as applications continue to use IPv4 addresses for indexing into tables,
        accounting, et cetera, it may be necessary to **coerce** IPv6 addresses into IPv4 addresses.
        This method does so by hashing 64 bits of the IPv6 address into `224.0.0.0/3` (64 bits
        into 29 bits):
        
        
          - If the IPv6 address contains an embedded IPv4 address, the function hashes that.
          - Otherwise, it hashes the upper 64 bits of the IPv6 address.
        
        
        A "coerced" IPv4 address is equivalent to itself.
        
        NOTE: This method is failsafe for security purposes: ALL IPv6 addresses (except localhost
        (::1)) are hashed to avoid the security risk associated with extracting an embedded IPv4
        address that might permit elevated privileges.

        Arguments
        - ip: InetAddress to "coerce"

        Returns
        - Inet4Address represented "coerced" address

        Since
        - 7.0
        """
        ...


    @staticmethod
    def coerceToInteger(ip: "InetAddress") -> int:
        """
        Returns an integer representing an IPv4 address regardless of whether the supplied argument is
        an IPv4 address or not.
        
        IPv6 addresses are **coerced** to IPv4 addresses before being converted to integers.
        
        As long as there are applications that assume that all IP addresses are IPv4 addresses and
        can therefore be converted safely to integers (for whatever purpose) this function can be used
        to handle IPv6 addresses as well until the application is suitably fixed.
        
        NOTE: an IPv6 address coerced to an IPv4 address can only be used for such purposes as
        rudimentary identification or indexing into a collection of real InetAddresses. They
        cannot be used as real addresses for the purposes of network communication.

        Arguments
        - ip: InetAddress to convert

        Returns
        - `int`, "coerced" if ip is not an IPv4 address

        Since
        - 7.0
        """
        ...


    @staticmethod
    def toBigInteger(address: "InetAddress") -> "BigInteger":
        """
        Returns a BigInteger representing the address.
        
        Unlike `coerceToInteger`, IPv6 addresses are not coerced to IPv4 addresses.

        Arguments
        - address: InetAddress to convert

        Returns
        - `BigInteger` representation of the address

        Since
        - 28.2
        """
        ...


    @staticmethod
    def fromInteger(address: int) -> "Inet4Address":
        """
        Returns an Inet4Address having the integer value specified by the argument.

        Arguments
        - address: `int`, the 32bit integer address to be converted

        Returns
        - Inet4Address equivalent of the argument
        """
        ...


    @staticmethod
    def fromIPv4BigInteger(address: "BigInteger") -> "Inet4Address":
        """
        Returns the `Inet4Address` corresponding to a given `BigInteger`.

        Arguments
        - address: BigInteger representing the IPv4 address

        Returns
        - Inet4Address representation of the given BigInteger

        Raises
        - IllegalArgumentException: if the BigInteger is not between 0 and 2^32-1

        Since
        - 28.2
        """
        ...


    @staticmethod
    def fromIPv6BigInteger(address: "BigInteger") -> "Inet6Address":
        """
        Returns the `Inet6Address` corresponding to a given `BigInteger`.

        Arguments
        - address: BigInteger representing the IPv6 address

        Returns
        - Inet6Address representation of the given BigInteger

        Raises
        - IllegalArgumentException: if the BigInteger is not between 0 and 2^128-1

        Since
        - 28.2
        """
        ...


    @staticmethod
    def fromLittleEndianByteArray(addr: list[int]) -> "InetAddress":
        """
        Returns an address from a **little-endian ordered** byte array (the opposite of what InetAddress.getByAddress expects).
        
        IPv4 address byte array must be 4 bytes long and IPv6 byte array must be 16 bytes long.

        Arguments
        - addr: the raw IP address in little-endian byte order

        Returns
        - an InetAddress object created from the raw IP address

        Raises
        - UnknownHostException: if IP address is of illegal length
        """
        ...


    @staticmethod
    def decrement(address: "InetAddress") -> "InetAddress":
        """
        Returns a new InetAddress that is one less than the passed in address. This method works for
        both IPv4 and IPv6 addresses.

        Arguments
        - address: the InetAddress to decrement

        Returns
        - a new InetAddress that is one less than the passed in address

        Raises
        - IllegalArgumentException: if InetAddress is at the beginning of its range

        Since
        - 18.0
        """
        ...


    @staticmethod
    def increment(address: "InetAddress") -> "InetAddress":
        """
        Returns a new InetAddress that is one more than the passed in address. This method works for
        both IPv4 and IPv6 addresses.

        Arguments
        - address: the InetAddress to increment

        Returns
        - a new InetAddress that is one more than the passed in address

        Raises
        - IllegalArgumentException: if InetAddress is at the end of its range

        Since
        - 10.0
        """
        ...


    @staticmethod
    def isMaximum(address: "InetAddress") -> bool:
        """
        Returns True if the InetAddress is either 255.255.255.255 for IPv4 or
        ffff:ffff:ffff:ffff:ffff:ffff:ffff:ffff for IPv6.

        Returns
        - True if the InetAddress is either 255.255.255.255 for IPv4 or
            ffff:ffff:ffff:ffff:ffff:ffff:ffff:ffff for IPv6

        Since
        - 10.0
        """
        ...


    class TeredoInfo:
        """
        A simple immutable data class to encapsulate the information to be found in a Teredo address.
        
        All of the fields in this class are encoded in various portions of the IPv6 address as part
        of the protocol. More protocols details can be found at: <a target="_parent"
        href="http://en.wikipedia.org/wiki/Teredo_tunneling">http://en.wikipedia.
        org/wiki/Teredo_tunneling</a>.
        
        The RFC can be found here: <a target="_parent" href="http://tools.ietf.org/html/rfc4380">RFC
        4380</a>.

        Since
        - 5.0
        """

        def __init__(self, server: "Inet4Address", client: "Inet4Address", port: int, flags: int):
            ...


        def getServer(self) -> "Inet4Address":
            ...


        def getClient(self) -> "Inet4Address":
            ...


        def getPort(self) -> int:
            ...


        def getFlags(self) -> int:
            ...



"""
Python module generated from Java source file org.bukkit.inventory.FurnaceInventory

Java source file obtained from artifact spigot-api version 1.21.4-R0.1-20250303.102353-42

Because this Python module is automatically generated, it may contain errors
and/or code that cannot be parsed. Please report these issues at
https://github.com/magicmq/docs-translator/issues
"""
from org.bukkit.block import Furnace
from org.bukkit.inventory import *
from typing import Any, Callable, Iterable, Tuple


class FurnaceInventory(Inventory):
    """
    Interface to the inventory of a Furnace.
    """

    def getResult(self) -> "ItemStack":
        """
        Get the current item in the result slot.

        Returns
        - The item
        """
        ...


    def getFuel(self) -> "ItemStack":
        """
        Get the current fuel.

        Returns
        - The item
        """
        ...


    def getSmelting(self) -> "ItemStack":
        """
        Get the item currently smelting.

        Returns
        - The item
        """
        ...


    def setFuel(self, stack: "ItemStack") -> None:
        """
        Set the current fuel.

        Arguments
        - stack: The item
        """
        ...


    def setResult(self, stack: "ItemStack") -> None:
        """
        Set the current item in the result slot.

        Arguments
        - stack: The item
        """
        ...


    def setSmelting(self, stack: "ItemStack") -> None:
        """
        Set the item currently smelting.

        Arguments
        - stack: The item
        """
        ...


    def getHolder(self) -> "Furnace":
        ...



"""
Python module generated from Java source file com.google.common.collect.FilteredMultimapValues

Java source file obtained from artifact guava version 33.3.1-jre

Because this Python module is automatically generated, it may contain errors
and/or code that cannot be parsed. Please report these issues at
https://github.com/magicmq/docs-translator/issues
"""
from com.google.common.annotations import GwtCompatible
from com.google.common.base import Objects
from com.google.common.base import Predicate
from com.google.common.base import Predicates
from com.google.common.collect import *
from com.google.j2objc.annotations import Weak
from java.util import AbstractCollection
from java.util import Iterator
from javax.annotation import CheckForNull
from org.checkerframework.checker.nullness.qual import Nullable
from typing import Any, Callable, Iterable, Tuple


class FilteredMultimapValues(AbstractCollection):
    """
    Implementation for FilteredMultimap.values().

    Author(s)
    - Louis Wasserman
    """

    def iterator(self) -> Iterator["V"]:
        ...


    def contains(self, o: "Object") -> bool:
        ...


    def size(self) -> int:
        ...


    def remove(self, o: "Object") -> bool:
        ...


    def removeAll(self, c: Iterable[Any]) -> bool:
        ...


    def retainAll(self, c: Iterable[Any]) -> bool:
        ...


    def clear(self) -> None:
        ...



"""
Python module generated from Java source file org.bukkit.plugin.TimedRegisteredListener

Java source file obtained from artifact spigot-api version 1.21.4-R0.1-20250303.102353-42

Because this Python module is automatically generated, it may contain errors
and/or code that cannot be parsed. Please report these issues at
https://github.com/magicmq/docs-translator/issues
"""
from org.bukkit.event import Event
from org.bukkit.event import EventException
from org.bukkit.event import EventPriority
from org.bukkit.event import Listener
from org.bukkit.plugin import *
from typing import Any, Callable, Iterable, Tuple


class TimedRegisteredListener(RegisteredListener):
    """
    Extends RegisteredListener to include timing information
    """

    def __init__(self, pluginListener: "Listener", eventExecutor: "EventExecutor", eventPriority: "EventPriority", registeredPlugin: "Plugin", listenCancelled: bool):
        ...


    def callEvent(self, event: "Event") -> None:
        ...


    def reset(self) -> None:
        """
        Resets the call count and total time for this listener
        """
        ...


    def getCount(self) -> int:
        """
        Gets the total times this listener has been called

        Returns
        - Times this listener has been called
        """
        ...


    def getTotalTime(self) -> int:
        """
        Gets the total time calls to this listener have taken

        Returns
        - Total time for all calls of this listener
        """
        ...


    def getEventClass(self) -> type["Event"]:
        """
        Gets the class of the events this listener handled. If it handled
        multiple classes of event, the closest shared superclass will be
        returned, such that for any event this listener has handled,
        `this.getEventClass().isAssignableFrom(event.getClass())`
        and no class `this.getEventClass().isAssignableFrom(clazz)
        && this.getEventClass() != clazz &&
        event.getClass().isAssignableFrom(clazz)` for all handled events.

        Returns
        - the event class handled by this RegisteredListener
        """
        ...


    def hasMultiple(self) -> bool:
        """
        Gets whether this listener has handled multiple events, such that for
        some two events, `eventA.getClass() != eventB.getClass()`.

        Returns
        - True if this listener has handled multiple events
        """
        ...



"""
Python module generated from Java source file org.bukkit.conversations.InactivityConversationCanceller

Java source file obtained from artifact spigot-api version 1.21.4-R0.1-20250303.102353-42

Because this Python module is automatically generated, it may contain errors
and/or code that cannot be parsed. Please report these issues at
https://github.com/magicmq/docs-translator/issues
"""
from org.bukkit.conversations import *
from org.bukkit.plugin import Plugin
from typing import Any, Callable, Iterable, Tuple


class InactivityConversationCanceller(ConversationCanceller):
    """
    An InactivityConversationCanceller will cancel a Conversation after
    a period of inactivity by the user.
    """

    def __init__(self, plugin: "Plugin", timeoutSeconds: int):
        """
        Creates an InactivityConversationCanceller.

        Arguments
        - plugin: The owning plugin.
        - timeoutSeconds: The number of seconds of inactivity to wait.
        """
        ...


    def setConversation(self, conversation: "Conversation") -> None:
        ...


    def cancelBasedOnInput(self, context: "ConversationContext", input: str) -> bool:
        ...


    def clone(self) -> "ConversationCanceller":
        ...



"""
Python module generated from Java source file org.bukkit.block.data.type.Chest

Java source file obtained from artifact spigot-api version 1.21.4-R0.1-20250303.102353-42

Because this Python module is automatically generated, it may contain errors
and/or code that cannot be parsed. Please report these issues at
https://github.com/magicmq/docs-translator/issues
"""
from enum import Enum
from org.bukkit.block.data import Directional
from org.bukkit.block.data import Waterlogged
from org.bukkit.block.data.type import *
from typing import Any, Callable, Iterable, Tuple


class Chest(Directional, Waterlogged):
    """
    'type' represents which part of a double chest this block is, or if it is a
    single chest.
    """

    def getType(self) -> "Type":
        """
        Gets the value of the 'type' property.

        Returns
        - the 'type' value
        """
        ...


    def setType(self, type: "Type") -> None:
        """
        Sets the value of the 'type' property.

        Arguments
        - type: the new 'type' value
        """
        ...


    class Type(Enum):
        """
        Type of this chest block.
        
        NB: Left and right are relative to the chest itself, i.e opposite to what
        a player placing the appropriate block would see.
        """

        SINGLE = 0
        """
        The chest is not linked to any others and contains only one 27 slot
        inventory.
        """
        LEFT = 1
        """
        The chest is the left hand side of a double chest and shares a 54
        block inventory with the chest to its right.
        """
        RIGHT = 2
        """
        The chest is the right hand side of a double chest and shares a 54
        block inventory with the chest to its left.
        """



"""
Python module generated from Java source file com.google.common.cache.IgnoreJRERequirement

Java source file obtained from artifact guava version 33.3.1-jre

Because this Python module is automatically generated, it may contain errors
and/or code that cannot be parsed. Please report these issues at
https://github.com/magicmq/docs-translator/issues
"""
from com.google.common.cache import *
from typing import Any, Callable, Iterable, Tuple


class IgnoreJRERequirement:
    """
    Disables Animal Sniffer's checking of compatibility with older versions of Java/Android.
    
    Each package's copy of this annotation needs to be listed in our `pom.xml`.
    """





"""
Python module generated from Java source file org.bukkit.entity.ThrowableProjectile

Java source file obtained from artifact spigot-api version 1.21.4-R0.1-20250303.102353-42

Because this Python module is automatically generated, it may contain errors
and/or code that cannot be parsed. Please report these issues at
https://github.com/magicmq/docs-translator/issues
"""
from org.bukkit.entity import *
from org.bukkit.inventory import ItemStack
from typing import Any, Callable, Iterable, Tuple


class ThrowableProjectile(Projectile):

    def getItem(self) -> "ItemStack":
        """
        Gets the ItemStack the thrown projectile will display.

        Returns
        - The thrown item display ItemStack
        """
        ...


    def setItem(self, item: "ItemStack") -> None:
        """
        Sets the display ItemStack for the thrown projectile.

        Arguments
        - item: ItemStack set to be displayed
        """
        ...



from .TempFileCreator import TempFileCreator
from .Resources import Resources
from .RecursiveDeleteOption import RecursiveDeleteOption
from .ReaderInputStream import ReaderInputStream
from .PatternFilenameFilter import PatternFilenameFilter
from .ParametricNullness import ParametricNullness
from .MultiReader import MultiReader
from .MultiInputStream import MultiInputStream
from .MoreFiles import MoreFiles
from .LittleEndianDataOutputStream import LittleEndianDataOutputStream
from .LittleEndianDataInputStream import LittleEndianDataInputStream
from .LineReader import LineReader
from .LineProcessor import LineProcessor
from .LineBuffer import LineBuffer
from .Java8Compatibility import Java8Compatibility
from .InsecureRecursiveDeleteException import InsecureRecursiveDeleteException
from .IgnoreJRERequirement import IgnoreJRERequirement
from .Flushables import Flushables
from .Files import Files
from .FileWriteMode import FileWriteMode
from .FileBackedOutputStream import FileBackedOutputStream
from .ElementTypesAreNonnullByDefault import ElementTypesAreNonnullByDefault
from .CountingOutputStream import CountingOutputStream
from .CountingInputStream import CountingInputStream
from .Closer import Closer
from .Closeables import Closeables
from .CharStreams import CharStreams
from .CharSource import CharSource
from .CharSink import CharSink
from .CharSequenceReader import CharSequenceReader
from .ByteStreams import ByteStreams
from .ByteSource import ByteSource
from .ByteSink import ByteSink
from .ByteProcessor import ByteProcessor
from .ByteArrayDataOutput import ByteArrayDataOutput
from .ByteArrayDataInput import ByteArrayDataInput
from .BaseEncoding import BaseEncoding
from .AppendableWriter import AppendableWriter


"""
Python module generated from Java source file java.lang.reflect.GenericDeclaration

Java source file obtained from artifact jdk version jdk

Because this Python module is automatically generated, it may contain errors
and/or code that cannot be parsed. Please report these issues at
https://github.com/magicmq/docs-translator/issues
"""
from java.lang.reflect import *
from typing import Any, Callable, Iterable, Tuple


class GenericDeclaration(AnnotatedElement):
    """
    A common interface for all entities that declare type variables.

    Since
    - 1.5
    """

    def getTypeParameters(self) -> list["TypeVariable"[Any]]:
        """
        Returns an array of `TypeVariable` objects that
        represent the type variables declared by the generic
        declaration represented by this `GenericDeclaration`
        object, in declaration order.  Returns an array of length 0 if
        the underlying generic declaration declares no type variables.

        Returns
        - an array of `TypeVariable` objects that represent
            the type variables declared by this generic declaration

        Raises
        - GenericSignatureFormatError: if the generic
            signature of this generic declaration does not conform to
            the format specified in
            <cite>The Java Virtual Machine Specification</cite>
        """
        ...



"""
Python module generated from Java source file com.google.common.collect.Sets

Java source file obtained from artifact guava version 33.3.1-jre

Because this Python module is automatically generated, it may contain errors
and/or code that cannot be parsed. Please report these issues at
https://github.com/magicmq/docs-translator/issues
"""
from com.google.common.annotations import GwtCompatible
from com.google.common.annotations import GwtIncompatible
from com.google.common.annotations import J2ktIncompatible
from com.google.common.base import Predicate
from com.google.common.base import Predicates
from com.google.common.collect import *
from com.google.common.collect.Collections2 import FilteredCollection
from com.google.common.math import IntMath
from com.google.errorprone.annotations import CanIgnoreReturnValue
from com.google.errorprone.annotations import DoNotCall
from com.google.errorprone.annotations.concurrent import LazyInit
from java.io import Serializable
from java.util import AbstractSet
from java.util import Arrays
from java.util import BitSet
from java.util import Collections
from java.util import Comparator
from java.util import EnumSet
from java.util import Iterator
from java.util import LinkedHashSet
from java.util import NavigableSet
from java.util import NoSuchElementException
from java.util import SortedSet
from java.util.concurrent import ConcurrentHashMap
from java.util.concurrent import CopyOnWriteArraySet
from java.util.function import Consumer
from java.util.stream import Collector
from java.util.stream import Stream
from javax.annotation import CheckForNull
from org.checkerframework.checker.nullness.qual import NonNull
from org.checkerframework.checker.nullness.qual import Nullable
from typing import Any, Callable, Iterable, Tuple


class Sets:
    """
    Static utility methods pertaining to Set instances. Also see this class's counterparts
    Lists, Maps and Queues.
    
    See the Guava User Guide article on <a href=
    "https://github.com/google/guava/wiki/CollectionUtilitiesExplained#sets">`Sets`</a>.

    Author(s)
    - Chris Povirk

    Since
    - 2.0
    """

    @staticmethod
    def immutableEnumSet(anElement: "E", *otherElements: Tuple["E", ...]) -> "ImmutableSet"["E"]:
        ...


    @staticmethod
    def immutableEnumSet(elements: Iterable["E"]) -> "ImmutableSet"["E"]:
        ...


    @staticmethod
    def toImmutableEnumSet() -> "Collector"["E", Any, "ImmutableSet"["E"]]:
        """
        Returns a `Collector` that accumulates the input elements into a new `ImmutableSet`
        with an implementation specialized for enums. Unlike ImmutableSet.toImmutableSet, the
        resulting set will iterate over elements in their enum definition order, not encounter order.

        Since
        - 21.0
        """
        ...


    @staticmethod
    def newEnumSet(iterable: Iterable["E"], elementType: type["E"]) -> "EnumSet"["E"]:
        """
        Returns a new, *mutable* `EnumSet` instance containing the given elements in their
        natural order. This method behaves identically to EnumSet.copyOf(Collection), but also
        accepts non-`Collection` iterables and empty iterables.
        """
        ...


    @staticmethod
    def newHashSet() -> set["E"]:
        """
        Creates a *mutable*, initially empty `HashSet` instance.
        
        **Note:** if mutability is not required, use ImmutableSet.of() instead. If `E` is an Enum type, use EnumSet.noneOf instead. Otherwise, strongly consider
        using a `LinkedHashSet` instead, at the cost of increased memory footprint, to get
        deterministic iteration behavior.
        
        **Note:** this method is now unnecessary and should be treated as deprecated. Instead,
        use the `HashSet` constructor directly, taking advantage of <a
        href="https://docs.oracle.com/javase/tutorial/java/generics/genTypeInference.html#type-inference-instantiation">"diamond"
        syntax</a>.
        """
        ...


    @staticmethod
    def newHashSet(*elements: Tuple["E", ...]) -> set["E"]:
        """
        Creates a *mutable* `HashSet` instance initially containing the given elements.
        
        **Note:** if elements are non-null and won't be added or removed after this point, use
        ImmutableSet.of() or ImmutableSet.copyOf(Object[]) instead. If `E` is an
        Enum type, use EnumSet.of(Enum, Enum[]) instead. Otherwise, strongly consider
        using a `LinkedHashSet` instead, at the cost of increased memory footprint, to get
        deterministic iteration behavior.
        
        This method is just a small convenience, either for `newHashSet(`Arrays.asList
        asList`(...))`, or for creating an empty set then calling Collections.addAll.
        This method is not actually very useful and will likely be deprecated in the future.
        """
        ...


    @staticmethod
    def newHashSet(elements: Iterable["E"]) -> set["E"]:
        """
        Creates a *mutable* `HashSet` instance containing the given elements. A very thin
        convenience for creating an empty set then calling Collection.addAll or Iterables.addAll.
        
        **Note:** if mutability is not required and the elements are non-null, use ImmutableSet.copyOf(Iterable) instead. (Or, change `elements` to be a FluentIterable and call `elements.toSet()`.)
        
        **Note:** if `E` is an Enum type, use .newEnumSet(Iterable, Class)
        instead.
        
        **Note:** if `elements` is a Collection, you don't need this method.
        Instead, use the `HashSet` constructor directly, taking advantage of <a
        href="https://docs.oracle.com/javase/tutorial/java/generics/genTypeInference.html#type-inference-instantiation">"diamond"
        syntax</a>.
        
        Overall, this method is not very useful and will likely be deprecated in the future.
        """
        ...


    @staticmethod
    def newHashSet(elements: Iterator["E"]) -> set["E"]:
        """
        Creates a *mutable* `HashSet` instance containing the given elements. A very thin
        convenience for creating an empty set and then calling Iterators.addAll.
        
        **Note:** if mutability is not required and the elements are non-null, use ImmutableSet.copyOf(Iterator) instead.
        
        **Note:** if `E` is an Enum type, you should create an EnumSet
        instead.
        
        Overall, this method is not very useful and will likely be deprecated in the future.
        """
        ...


    @staticmethod
    def newHashSetWithExpectedSize(expectedSize: int) -> set["E"]:
        """
        Returns a new hash set using the smallest initial table size that can hold `expectedSize`
        elements without resizing. Note that this is not what HashSet.HashSet(int) does, but it
        is what most users want and expect it to do.
        
        This behavior can't be broadly guaranteed, but has been tested with OpenJDK 1.7 and 1.8.

        Arguments
        - expectedSize: the number of elements you expect to add to the returned set

        Returns
        - a new, empty hash set with enough capacity to hold `expectedSize` elements
            without resizing

        Raises
        - IllegalArgumentException: if `expectedSize` is negative
        """
        ...


    @staticmethod
    def newConcurrentHashSet() -> set["E"]:
        """
        Creates a thread-safe set backed by a hash map. The set is backed by a ConcurrentHashMap instance, and thus carries the same concurrency guarantees.
        
        Unlike `HashSet`, this class does NOT allow `null` to be used as an element. The
        set is serializable.

        Returns
        - a new, empty thread-safe `Set`

        Since
        - 15.0
        """
        ...


    @staticmethod
    def newConcurrentHashSet(elements: Iterable["E"]) -> set["E"]:
        """
        Creates a thread-safe set backed by a hash map and containing the given elements. The set is
        backed by a ConcurrentHashMap instance, and thus carries the same concurrency
        guarantees.
        
        Unlike `HashSet`, this class does NOT allow `null` to be used as an element. The
        set is serializable.

        Arguments
        - elements: the elements that the set should contain

        Returns
        - a new thread-safe set containing those elements (minus duplicates)

        Raises
        - NullPointerException: if `elements` or any of its contents is null

        Since
        - 15.0
        """
        ...


    @staticmethod
    def newLinkedHashSet() -> "LinkedHashSet"["E"]:
        """
        Creates a *mutable*, empty `LinkedHashSet` instance.
        
        **Note:** if mutability is not required, use ImmutableSet.of() instead.
        
        **Note:** this method is now unnecessary and should be treated as deprecated. Instead,
        use the `LinkedHashSet` constructor directly, taking advantage of <a
        href="https://docs.oracle.com/javase/tutorial/java/generics/genTypeInference.html#type-inference-instantiation">"diamond"
        syntax</a>.

        Returns
        - a new, empty `LinkedHashSet`
        """
        ...


    @staticmethod
    def newLinkedHashSet(elements: Iterable["E"]) -> "LinkedHashSet"["E"]:
        """
        Creates a *mutable* `LinkedHashSet` instance containing the given elements in order.
        
        **Note:** if mutability is not required and the elements are non-null, use ImmutableSet.copyOf(Iterable) instead.
        
        **Note:** if `elements` is a Collection, you don't need this method.
        Instead, use the `LinkedHashSet` constructor directly, taking advantage of <a
        href="https://docs.oracle.com/javase/tutorial/java/generics/genTypeInference.html#type-inference-instantiation">"diamond"
        syntax</a>.
        
        Overall, this method is not very useful and will likely be deprecated in the future.

        Arguments
        - elements: the elements that the set should contain, in order

        Returns
        - a new `LinkedHashSet` containing those elements (minus duplicates)
        """
        ...


    @staticmethod
    def newLinkedHashSetWithExpectedSize(expectedSize: int) -> "LinkedHashSet"["E"]:
        """
        Creates a `LinkedHashSet` instance, with a high enough "initial capacity" that it
        *should* hold `expectedSize` elements without growth. This behavior cannot be
        broadly guaranteed, but it is observed to be True for OpenJDK 1.7. It also can't be guaranteed
        that the method isn't inadvertently *oversizing* the returned set.

        Arguments
        - expectedSize: the number of elements you expect to add to the returned set

        Returns
        - a new, empty `LinkedHashSet` with enough capacity to hold `expectedSize`
            elements without resizing

        Raises
        - IllegalArgumentException: if `expectedSize` is negative

        Since
        - 11.0
        """
        ...


    @staticmethod
    def newTreeSet() -> set["E"]:
        """
        Creates a *mutable*, empty `TreeSet` instance sorted by the natural sort ordering of
        its elements.
        
        **Note:** if mutability is not required, use ImmutableSortedSet.of() instead.
        
        **Note:** this method is now unnecessary and should be treated as deprecated. Instead,
        use the `TreeSet` constructor directly, taking advantage of <a
        href="https://docs.oracle.com/javase/tutorial/java/generics/genTypeInference.html#type-inference-instantiation">"diamond"
        syntax</a>.

        Returns
        - a new, empty `TreeSet`
        """
        ...


    @staticmethod
    def newTreeSet(elements: Iterable["E"]) -> set["E"]:
        """
        Creates a *mutable* `TreeSet` instance containing the given elements sorted by their
        natural ordering.
        
        **Note:** if mutability is not required, use ImmutableSortedSet.copyOf(Iterable)
        instead.
        
        **Note:** If `elements` is a `SortedSet` with an explicit comparator, this
        method has different behavior than TreeSet.TreeSet(SortedSet), which returns a `TreeSet` with that comparator.
        
        **Note:** this method is now unnecessary and should be treated as deprecated. Instead,
        use the `TreeSet` constructor directly, taking advantage of <a
        href="https://docs.oracle.com/javase/tutorial/java/generics/genTypeInference.html#type-inference-instantiation">"diamond"
        syntax</a>.
        
        This method is just a small convenience for creating an empty set and then calling Iterables.addAll. This method is not very useful and will likely be deprecated in the future.

        Arguments
        - elements: the elements that the set should contain

        Returns
        - a new `TreeSet` containing those elements (minus duplicates)
        """
        ...


    @staticmethod
    def newTreeSet(comparator: "Comparator"["E"]) -> set["E"]:
        """
        Creates a *mutable*, empty `TreeSet` instance with the given comparator.
        
        **Note:** if mutability is not required, use `ImmutableSortedSet.orderedBy(comparator).build()` instead.
        
        **Note:** this method is now unnecessary and should be treated as deprecated. Instead,
        use the `TreeSet` constructor directly, taking advantage of <a
        href="https://docs.oracle.com/javase/tutorial/java/generics/genTypeInference.html#type-inference-instantiation">"diamond"
        syntax</a>. One caveat to this is that the `TreeSet` constructor uses a null `Comparator` to mean "natural ordering," whereas this factory rejects null. Clean your code
        accordingly.

        Arguments
        - comparator: the comparator to use to sort the set

        Returns
        - a new, empty `TreeSet`

        Raises
        - NullPointerException: if `comparator` is null
        """
        ...


    @staticmethod
    def newIdentityHashSet() -> set["E"]:
        """
        Creates an empty `Set` that uses identity to determine equality. It compares object
        references, instead of calling `equals`, to determine whether a provided object matches
        an element in the set. For example, `contains` returns `False` when passed an
        object that equals a set member, but isn't the same instance. This behavior is similar to the
        way `IdentityHashMap` handles key lookups.

        Since
        - 8.0
        """
        ...


    @staticmethod
    def newCopyOnWriteArraySet() -> "CopyOnWriteArraySet"["E"]:
        """
        Creates an empty `CopyOnWriteArraySet` instance.
        
        **Note:** if you need an immutable empty Set, use Collections.emptySet
        instead.

        Returns
        - a new, empty `CopyOnWriteArraySet`

        Since
        - 12.0
        """
        ...


    @staticmethod
    def newCopyOnWriteArraySet(elements: Iterable["E"]) -> "CopyOnWriteArraySet"["E"]:
        """
        Creates a `CopyOnWriteArraySet` instance containing the given elements.

        Arguments
        - elements: the elements that the set should contain, in order

        Returns
        - a new `CopyOnWriteArraySet` containing those elements

        Since
        - 12.0
        """
        ...


    @staticmethod
    def complementOf(collection: Iterable["E"]) -> "EnumSet"["E"]:
        """
        Creates an `EnumSet` consisting of all enum values that are not in the specified
        collection. If the collection is an EnumSet, this method has the same behavior as
        EnumSet.complementOf. Otherwise, the specified collection must contain at least one
        element, in order to determine the element type. If the collection could be empty, use .complementOf(Collection, Class) instead of this method.

        Arguments
        - collection: the collection whose complement should be stored in the enum set

        Returns
        - a new, modifiable `EnumSet` containing all values of the enum that aren't present
            in the given collection

        Raises
        - IllegalArgumentException: if `collection` is not an `EnumSet` instance and
            contains no elements
        """
        ...


    @staticmethod
    def complementOf(collection: Iterable["E"], type: type["E"]) -> "EnumSet"["E"]:
        """
        Creates an `EnumSet` consisting of all enum values that are not in the specified
        collection. This is equivalent to EnumSet.complementOf, but can act on any input
        collection, as long as the elements are of enum type.

        Arguments
        - collection: the collection whose complement should be stored in the `EnumSet`
        - type: the type of the elements in the set

        Returns
        - a new, modifiable `EnumSet` initially containing all the values of the enum not
            present in the given collection
        """
        ...


    @staticmethod
    def newSetFromMap(map: dict["E", "Boolean"]) -> set["E"]:
        """
        Returns a set backed by the specified map. The resulting set displays the same ordering,
        concurrency, and performance characteristics as the backing map. In essence, this factory
        method provides a Set implementation corresponding to any Map implementation.
        There is no need to use this method on a Map implementation that already has a
        corresponding Set implementation (such as java.util.HashMap or java.util.TreeMap).
        
        Each method invocation on the set returned by this method results in exactly one method
        invocation on the backing map or its `keySet` view, with one exception. The `addAll` method is implemented as a sequence of `put` invocations on the backing map.
        
        The specified map must be empty at the time this method is invoked, and should not be
        accessed directly after this method returns. These conditions are ensured if the map is created
        empty, passed directly to this method, and no reference to the map is retained, as illustrated
        in the following code fragment:
        
        ````Set<Object> identityHashSet = Sets.newSetFromMap(
            new IdentityHashMap<Object, Boolean>());````
        
        The returned set is serializable if the backing map is.

        Arguments
        - map: the backing map

        Returns
        - the set backed by the map

        Raises
        - IllegalArgumentException: if `map` is not empty

        Deprecated
        - Use Collections.newSetFromMap instead.
        """
        ...


    @staticmethod
    def union(set1: set["E"], set2: set["E"]) -> "SetView"["E"]:
        """
        Returns an unmodifiable **view** of the union of two sets. The returned set contains all
        elements that are contained in either backing set. Iterating over the returned set iterates
        first over all the elements of `set1`, then over each element of `set2`, in order,
        that is not contained in `set1`.
        
        Results are undefined if `set1` and `set2` are sets based on different
        equivalence relations, for example if `set1` is a HashSet and `set2` is a
        TreeSet or the Map.keySet of an `IdentityHashMap`.
        """
        ...


    @staticmethod
    def intersection(set1: set["E"], set2: set[Any]) -> "SetView"["E"]:
        """
        Returns an unmodifiable **view** of the intersection of two sets. The returned set contains
        all elements that are contained by both backing sets. The iteration order of the returned set
        matches that of `set1`.
        
        Results are undefined if `set1` and `set2` are sets based on different
        equivalence relations, for example if `set1` is a HashSet and `set2` is a
        TreeSet or the Map.keySet of an `IdentityHashMap`.
        
        **Note:** The returned view performs slightly better when `set1` is the smaller of
        the two sets. If you have reason to believe one of your sets will generally be smaller than the
        other, pass it first. Unfortunately, since this method sets the generic type of the returned
        set based on the type of the first set passed, this could in rare cases force you to make a
        cast, for example:
        
        ````Set<Object> aFewBadObjects = ...
        Set<String> manyBadStrings = ...
        
        // impossible for a non-String to be in the intersection
        SuppressWarnings("unchecked")
        Set<String> badStrings = (Set) Sets.intersection(
            aFewBadObjects, manyBadStrings);````
        
        This is unfortunate, but should come up only very rarely.
        """
        ...


    @staticmethod
    def difference(set1: set["E"], set2: set[Any]) -> "SetView"["E"]:
        """
        Returns an unmodifiable **view** of the difference of two sets. The returned set contains
        all elements that are contained by `set1` and not contained by `set2`. `set2`
        may also contain elements not present in `set1`; these are simply ignored. The iteration
        order of the returned set matches that of `set1`.
        
        Results are undefined if `set1` and `set2` are sets based on different
        equivalence relations, for example if `set1` is a HashSet and `set2` is a
        TreeSet or the Map.keySet of an `IdentityHashMap`.
        """
        ...


    @staticmethod
    def symmetricDifference(set1: set["E"], set2: set["E"]) -> "SetView"["E"]:
        """
        Returns an unmodifiable **view** of the symmetric difference of two sets. The returned set
        contains all elements that are contained in either `set1` or `set2` but not in
        both. The iteration order of the returned set is undefined.
        
        Results are undefined if `set1` and `set2` are sets based on different
        equivalence relations, for example if `set1` is a HashSet and `set2` is a
        TreeSet or the Map.keySet of an `IdentityHashMap`.

        Since
        - 3.0
        """
        ...


    @staticmethod
    def filter(unfiltered: set["E"], predicate: "Predicate"["E"]) -> set["E"]:
        ...


    @staticmethod
    def filter(unfiltered: "SortedSet"["E"], predicate: "Predicate"["E"]) -> "SortedSet"["E"]:
        """
        Returns the elements of a `SortedSet`, `unfiltered`, that satisfy a predicate. The
        returned set is a live view of `unfiltered`; changes to one affect the other.
        
        The resulting set's iterator does not support `remove()`, but all other set methods
        are supported. When given an element that doesn't satisfy the predicate, the set's `add()` and `addAll()` methods throw an IllegalArgumentException. When methods
        such as `removeAll()` and `clear()` are called on the filtered set, only elements
        that satisfy the filter will be removed from the underlying set.
        
        The returned set isn't threadsafe or serializable, even if `unfiltered` is.
        
        Many of the filtered set's methods, such as `size()`, iterate across every element in
        the underlying set and determine which elements satisfy the filter. When a live view is
        *not* needed, it may be faster to copy `Iterables.filter(unfiltered, predicate)` and
        use the copy.
        
        **Warning:** `predicate` must be *consistent with equals*, as documented at
        Predicate.apply. Do not provide a predicate such as `Predicates.instanceOf(ArrayList.class)`, which is inconsistent with equals. (See Iterables.filter(Iterable, Class) for related functionality.)

        Since
        - 11.0
        """
        ...


    @staticmethod
    def filter(unfiltered: "NavigableSet"["E"], predicate: "Predicate"["E"]) -> "NavigableSet"["E"]:
        """
        Returns the elements of a `NavigableSet`, `unfiltered`, that satisfy a predicate.
        The returned set is a live view of `unfiltered`; changes to one affect the other.
        
        The resulting set's iterator does not support `remove()`, but all other set methods
        are supported. When given an element that doesn't satisfy the predicate, the set's `add()` and `addAll()` methods throw an IllegalArgumentException. When methods
        such as `removeAll()` and `clear()` are called on the filtered set, only elements
        that satisfy the filter will be removed from the underlying set.
        
        The returned set isn't threadsafe or serializable, even if `unfiltered` is.
        
        Many of the filtered set's methods, such as `size()`, iterate across every element in
        the underlying set and determine which elements satisfy the filter. When a live view is
        *not* needed, it may be faster to copy `Iterables.filter(unfiltered, predicate)` and
        use the copy.
        
        **Warning:** `predicate` must be *consistent with equals*, as documented at
        Predicate.apply. Do not provide a predicate such as `Predicates.instanceOf(ArrayList.class)`, which is inconsistent with equals. (See Iterables.filter(Iterable, Class) for related functionality.)

        Since
        - 14.0
        """
        ...


    @staticmethod
    def cartesianProduct(sets: list[set["B"]]) -> set[list["B"]]:
        """
        Returns every possible list that can be formed by choosing one element from each of the given
        sets in order; the "n-ary <a href="http://en.wikipedia.org/wiki/Cartesian_product">Cartesian
        product</a>" of the sets. For example:
        
        ````Sets.cartesianProduct(ImmutableList.of(
            ImmutableSet.of(1, 2),
            ImmutableSet.of("A", "B", "C")))````
        
        returns a set containing six lists:
        
        
          - `ImmutableList.of(1, "A")`
          - `ImmutableList.of(1, "B")`
          - `ImmutableList.of(1, "C")`
          - `ImmutableList.of(2, "A")`
          - `ImmutableList.of(2, "B")`
          - `ImmutableList.of(2, "C")`
        
        
        The result is guaranteed to be in the "traditional", lexicographical order for Cartesian
        products that you would get from nesting for loops:
        
        ````for (B b0 : sets.get(0)) {
          for (B b1 : sets.get(1)) {
            ...
            ImmutableList<B> tuple = ImmutableList.of(b0, b1, ...);
            // operate on tuple`
        }
        }```
        
        Note that if any input set is empty, the Cartesian product will also be empty. If no sets at
        all are provided (an empty list), the resulting Cartesian product has one element, an empty
        list (counter-intuitive, but mathematically consistent).
        
        *Performance notes:* while the cartesian product of sets of size `m, n, p` is a
        set of size `m x n x p`, its actual memory consumption is much smaller. When the
        cartesian set is constructed, the input sets are merely copied. Only as the resulting set is
        iterated are the individual lists created, and these are not retained after iteration.
        
        Type `<B>`: any common base class shared by all axes (often just Object)

        Arguments
        - sets: the sets to choose elements from, in the order that the elements chosen from those
            sets should appear in the resulting lists

        Returns
        - the Cartesian product, as an immutable set containing immutable lists

        Raises
        - NullPointerException: if `sets`, any one of the `sets`, or any element of a
            provided set is null
        - IllegalArgumentException: if the cartesian product size exceeds the `int` range

        Since
        - 2.0
        """
        ...


    @staticmethod
    def cartesianProduct(*sets: Tuple[set["B"], ...]) -> set[list["B"]]:
        """
        Returns every possible list that can be formed by choosing one element from each of the given
        sets in order; the "n-ary <a href="http://en.wikipedia.org/wiki/Cartesian_product">Cartesian
        product</a>" of the sets. For example:
        
        ````Sets.cartesianProduct(
            ImmutableSet.of(1, 2),
            ImmutableSet.of("A", "B", "C"))````
        
        returns a set containing six lists:
        
        
          - `ImmutableList.of(1, "A")`
          - `ImmutableList.of(1, "B")`
          - `ImmutableList.of(1, "C")`
          - `ImmutableList.of(2, "A")`
          - `ImmutableList.of(2, "B")`
          - `ImmutableList.of(2, "C")`
        
        
        The result is guaranteed to be in the "traditional", lexicographical order for Cartesian
        products that you would get from nesting for loops:
        
        ````for (B b0 : sets.get(0)) {
          for (B b1 : sets.get(1)) {
            ...
            ImmutableList<B> tuple = ImmutableList.of(b0, b1, ...);
            // operate on tuple`
        }
        }```
        
        Note that if any input set is empty, the Cartesian product will also be empty. If no sets at
        all are provided (an empty list), the resulting Cartesian product has one element, an empty
        list (counter-intuitive, but mathematically consistent).
        
        *Performance notes:* while the cartesian product of sets of size `m, n, p` is a
        set of size `m x n x p`, its actual memory consumption is much smaller. When the
        cartesian set is constructed, the input sets are merely copied. Only as the resulting set is
        iterated are the individual lists created, and these are not retained after iteration.
        
        Type `<B>`: any common base class shared by all axes (often just Object)

        Arguments
        - sets: the sets to choose elements from, in the order that the elements chosen from those
            sets should appear in the resulting lists

        Returns
        - the Cartesian product, as an immutable set containing immutable lists

        Raises
        - NullPointerException: if `sets`, any one of the `sets`, or any element of a
            provided set is null
        - IllegalArgumentException: if the cartesian product size exceeds the `int` range

        Since
        - 2.0
        """
        ...


    @staticmethod
    def powerSet(set: set["E"]) -> set[set["E"]]:
        """
        Returns the set of all possible subsets of `set`. For example, `powerSet(ImmutableSet.of(1, 2))` returns the set `{{`, {1}, {2}, {1, 2}}}.
        
        Elements appear in these subsets in the same iteration order as they appeared in the input
        set. The order in which these subsets appear in the outer set is undefined. Note that the power
        set of the empty set is not the empty set, but a one-element set containing the empty set.
        
        The returned set and its constituent sets use `equals` to decide whether two elements
        are identical, even if the input set uses a different concept of equivalence.
        
        *Performance notes:* while the power set of a set with size `n` is of size `2^n`, its memory usage is only `O(n)`. When the power set is constructed, the input set
        is merely copied. Only as the power set is iterated are the individual subsets created, and
        these subsets themselves occupy only a small constant amount of memory.

        Arguments
        - set: the set of elements to construct a power set from

        Returns
        - the power set, as an immutable set of immutable sets

        Raises
        - IllegalArgumentException: if `set` has more than 30 unique elements (causing the
            power set size to exceed the `int` range)
        - NullPointerException: if `set` is or contains `null`

        See
        - <a href="http://en.wikipedia.org/wiki/Power_set">Power set article at Wikipedia</a>

        Since
        - 4.0
        """
        ...


    @staticmethod
    def combinations(set: set["E"], size: int) -> set[set["E"]]:
        """
        Returns the set of all subsets of `set` of size `size`. For example, `combinations(ImmutableSet.of(1, 2, 3), 2)` returns the set `{{1, 2`, {1, 3}, {2, 3}}}.
        
        Elements appear in these subsets in the same iteration order as they appeared in the input
        set. The order in which these subsets appear in the outer set is undefined.
        
        The returned set and its constituent sets use `equals` to decide whether two elements
        are identical, even if the input set uses a different concept of equivalence.
        
        *Performance notes:* the memory usage of the returned set is only `O(n)`. When
        the result set is constructed, the input set is merely copied. Only as the result set is
        iterated are the individual subsets created. Each of these subsets occupies an additional O(n)
        memory but only for as long as the user retains a reference to it. That is, the set returned by
        `combinations` does not retain the individual subsets.

        Arguments
        - set: the set of elements to take combinations of
        - size: the number of elements per combination

        Returns
        - the set of all combinations of `size` elements from `set`

        Raises
        - IllegalArgumentException: if `size` is not between 0 and `set.size()`
            inclusive
        - NullPointerException: if `set` is or contains `null`

        Since
        - 23.0
        """
        ...


    @staticmethod
    def unmodifiableNavigableSet(set: "NavigableSet"["E"]) -> "NavigableSet"["E"]:
        """
        Returns an unmodifiable view of the specified navigable set. This method allows modules to
        provide users with "read-only" access to internal navigable sets. Query operations on the
        returned set "read through" to the specified set, and attempts to modify the returned set,
        whether direct or via its collection views, result in an `UnsupportedOperationException`.
        
        The returned navigable set will be serializable if the specified navigable set is
        serializable.
        
        **Java 8+ users and later:** Prefer Collections.unmodifiableNavigableSet.

        Arguments
        - set: the navigable set for which an unmodifiable view is to be returned

        Returns
        - an unmodifiable view of the specified navigable set

        Since
        - 12.0
        """
        ...


    @staticmethod
    def synchronizedNavigableSet(navigableSet: "NavigableSet"["E"]) -> "NavigableSet"["E"]:
        """
        Returns a synchronized (thread-safe) navigable set backed by the specified navigable set. In
        order to guarantee serial access, it is critical that **all** access to the backing
        navigable set is accomplished through the returned navigable set (or its views).
        
        It is imperative that the user manually synchronize on the returned sorted set when
        iterating over it or any of its `descendingSet`, `subSet`, `headSet`, or
        `tailSet` views.
        
        ````NavigableSet<E> set = synchronizedNavigableSet(new TreeSet<E>());
         ...
        synchronized (set) {
          // Must be in the synchronized block
          Iterator<E> it = set.iterator();
          while (it.hasNext()) {
            foo(it.next());`
        }
        }```
        
        or:
        
        ````NavigableSet<E> set = synchronizedNavigableSet(new TreeSet<E>());
        NavigableSet<E> set2 = set.descendingSet().headSet(foo);
         ...
        synchronized (set) { // Note: set, not set2!!!
          // Must be in the synchronized block
          Iterator<E> it = set2.descendingIterator();
          while (it.hasNext())
            foo(it.next());`
        }
        }```
        
        Failure to follow this advice may result in non-deterministic behavior.
        
        The returned navigable set will be serializable if the specified navigable set is
        serializable.
        
        **Java 8+ users and later:** Prefer Collections.synchronizedNavigableSet.

        Arguments
        - navigableSet: the navigable set to be "wrapped" in a synchronized navigable set.

        Returns
        - a synchronized view of the specified navigable set.

        Since
        - 13.0
        """
        ...


    @staticmethod
    def subSet(set: "NavigableSet"["K"], range: "Range"["K"]) -> "NavigableSet"["K"]:
        """
        Returns a view of the portion of `set` whose elements are contained by `range`.
        
        This method delegates to the appropriate methods of NavigableSet (namely NavigableSet.subSet(Object, boolean, Object, boolean) subSet(), NavigableSet.tailSet(Object, boolean) tailSet(), and NavigableSet.headSet(Object,
        boolean) headSet()) to actually construct the view. Consult these methods for a full
        description of the returned view's behavior.
        
        **Warning:** `Range`s always represent a range of values using the values' natural
        ordering. `NavigableSet` on the other hand can specify a custom ordering via a Comparator, which can violate the natural ordering. Using this method (or in general using
        `Range`) with unnaturally-ordered sets can lead to unexpected and undefined behavior.

        Since
        - 20.0
        """
        ...


    class SetView(AbstractSet):
        """
        An unmodifiable view of a set which may be backed by other sets; this view will change as the
        backing sets do. Contains methods to copy the data into a new set which will then remain
        stable. There is usually no reason to retain a reference of type `SetView`; typically,
        you either use it as a plain Set, or immediately invoke .immutableCopy or
        .copyInto and forget the `SetView` itself.

        Since
        - 2.0
        """

        def immutableCopy(self) -> "ImmutableSet"["E"]:
            """
            Returns an immutable copy of the current contents of this set view. Does not support null
            elements.
            
            **Warning:** this may have unexpected results if a backing set of this view uses a
            nonstandard notion of equivalence, for example if it is a TreeSet using a comparator
            that is inconsistent with Object.equals(Object).
            """
            ...


        def copyInto(self, set: "S") -> "S":
            ...


        def add(self, e: "E") -> bool:
            """
            Guaranteed to throw an exception and leave the collection unmodified.

            Raises
            - UnsupportedOperationException: always

            Deprecated
            - Unsupported operation.
            """
            ...


        def remove(self, object: "Object") -> bool:
            """
            Guaranteed to throw an exception and leave the collection unmodified.

            Raises
            - UnsupportedOperationException: always

            Deprecated
            - Unsupported operation.
            """
            ...


        def addAll(self, newElements: Iterable["E"]) -> bool:
            """
            Guaranteed to throw an exception and leave the collection unmodified.

            Raises
            - UnsupportedOperationException: always

            Deprecated
            - Unsupported operation.
            """
            ...


        def removeAll(self, oldElements: Iterable[Any]) -> bool:
            """
            Guaranteed to throw an exception and leave the collection unmodified.

            Raises
            - UnsupportedOperationException: always

            Deprecated
            - Unsupported operation.
            """
            ...


        def removeIf(self, filter: "java.util.function.Predicate"["E"]) -> bool:
            """
            Guaranteed to throw an exception and leave the collection unmodified.

            Raises
            - UnsupportedOperationException: always

            Deprecated
            - Unsupported operation.
            """
            ...


        def retainAll(self, elementsToKeep: Iterable[Any]) -> bool:
            """
            Guaranteed to throw an exception and leave the collection unmodified.

            Raises
            - UnsupportedOperationException: always

            Deprecated
            - Unsupported operation.
            """
            ...


        def clear(self) -> None:
            """
            Guaranteed to throw an exception and leave the collection unmodified.

            Raises
            - UnsupportedOperationException: always

            Deprecated
            - Unsupported operation.
            """
            ...


        def iterator(self) -> "UnmodifiableIterator"["E"]:
            """
            Scope the return type to UnmodifiableIterator to ensure this is an unmodifiable view.

            Since
            - 20.0 (present with return type Iterator since 2.0)
            """
            ...



"""
Python module generated from Java source file org.bukkit.event.inventory.HopperInventorySearchEvent

Java source file obtained from artifact spigot-api version 1.21.4-R0.1-20250303.102353-42

Because this Python module is automatically generated, it may contain errors
and/or code that cannot be parsed. Please report these issues at
https://github.com/magicmq/docs-translator/issues
"""
from enum import Enum
from org.bukkit.block import Block
from org.bukkit.event import HandlerList
from org.bukkit.event.block import BlockEvent
from org.bukkit.event.inventory import *
from org.bukkit.inventory import Inventory
from typing import Any, Callable, Iterable, Tuple


class HopperInventorySearchEvent(BlockEvent):
    """
    Event that gets called each time a Hopper attempts to find its
    source/attached containers.
    """

    def __init__(self, inventory: "Inventory", containerType: "ContainerType", hopper: "Block", searchBlock: "Block"):
        ...


    def setInventory(self, inventory: "Inventory") -> None:
        """
        Set the Inventory that the Hopper will use for its
        source/attached Container.

        Arguments
        - inventory: the inventory to use
        """
        ...


    def getInventory(self) -> "Inventory":
        """
        Gets the Inventory that the Hopper will use for its
        source/attached Container.

        Returns
        - the inventory which will be used
        """
        ...


    def getContainerType(self) -> "ContainerType":
        """
        Gets the Container type the Hopper is searching for.

        Returns
        - the container type being searched for
        """
        ...


    def getSearchBlock(self) -> "Block":
        """
        Gets the Block that is being searched for an inventory.

        Returns
        - block being searched for an inventory
        """
        ...


    def getHandlers(self) -> "HandlerList":
        ...


    @staticmethod
    def getHandlerList() -> "HandlerList":
        ...


    class ContainerType(Enum):

        SOURCE = 0
        """
        The source container the hopper is looking for.
        
        This is the Inventory above the Hopper where it extracts items from.
        """
        DESTINATION = 1
        """
        The container the hopper is attached to.
        
        This is the Inventory the Hopper pushes items into.
        """



"""
Python module generated from Java source file com.google.gson.internal.bind.util.ISO8601Utils

Java source file obtained from artifact gson version 2.11.0

Because this Python module is automatically generated, it may contain errors
and/or code that cannot be parsed. Please report these issues at
https://github.com/magicmq/docs-translator/issues
"""
from com.google.gson.internal.bind.util import *
from java.text import ParseException
from java.text import ParsePosition
from java.util import Calendar
from java.util import Date
from java.util import GregorianCalendar
from java.util import Locale
from java.util import TimeZone
from typing import Any, Callable, Iterable, Tuple


class ISO8601Utils:

    @staticmethod
    def format(date: "Date") -> str:
        """
        Format a date into 'yyyy-MM-ddThh:mm:ssZ' (default timezone, no milliseconds precision)

        Arguments
        - date: the date to format

        Returns
        - the date formatted as 'yyyy-MM-ddThh:mm:ssZ'
        """
        ...


    @staticmethod
    def format(date: "Date", millis: bool) -> str:
        """
        Format a date into 'yyyy-MM-ddThh:mm:ss[.sss]Z' (GMT timezone)

        Arguments
        - date: the date to format
        - millis: True to include millis precision otherwise False

        Returns
        - the date formatted as 'yyyy-MM-ddThh:mm:ss[.sss]Z'
        """
        ...


    @staticmethod
    def format(date: "Date", millis: bool, tz: "TimeZone") -> str:
        """
        Format date into yyyy-MM-ddThh:mm:ss[.sss][Z|[+-]hh:mm]

        Arguments
        - date: the date to format
        - millis: True to include millis precision otherwise False
        - tz: timezone to use for the formatting (UTC will produce 'Z')

        Returns
        - the date formatted as yyyy-MM-ddThh:mm:ss[.sss][Z|[+-]hh:mm]
        """
        ...


    @staticmethod
    def parse(date: str, pos: "ParsePosition") -> "Date":
        """
        Parse a date from ISO-8601 formatted string. It expects a format
        [yyyy-MM-dd|yyyyMMdd][T(hh:mm[:ss[.sss]]|hhmm[ss[.sss]])]?[Z|[+-]hh[:mm]]]

        Arguments
        - date: ISO string to parse in the appropriate format.
        - pos: The position to start parsing from, updated to where parsing stopped.

        Returns
        - the parsed date

        Raises
        - ParseException: if the date is not in the appropriate format
        """
        ...



from .Type import Type
from .AccessibleObject import AccessibleObject
from .Field import Field
from .GenericArrayType import GenericArrayType
from .ParameterizedType import ParameterizedType
from .TypeVariable import TypeVariable
from .WildcardType import WildcardType
from .Method import Method
from .Modifier import Modifier
from .Constructor import Constructor
from .InvocationTargetException import InvocationTargetException
from .Array import Array
from .GenericDeclaration import GenericDeclaration
from .Member import Member
from .InvocationHandler import InvocationHandler
from .Proxy import Proxy
from .AnnotatedElement import AnnotatedElement
from .AnnotatedType import AnnotatedType


"""
Python module generated from Java source file org.yaml.snakeyaml.comments.CommentType

Java source file obtained from artifact snakeyaml version 2.2

Because this Python module is automatically generated, it may contain errors
and/or code that cannot be parsed. Please report these issues at
https://github.com/magicmq/docs-translator/issues
"""
from enum import Enum
from org.yaml.snakeyaml.comments import *
from typing import Any, Callable, Iterable, Tuple


class CommentType(Enum):
    """
    The type of comment line.
    """

    BLANK_LINE = 0
    BLOCK = 1
    IN_LINE = 2



"""
Python module generated from Java source file org.bukkit.conversations.ConversationAbandonedEvent

Java source file obtained from artifact spigot-api version 1.21.4-R0.1-20250303.102353-42

Because this Python module is automatically generated, it may contain errors
and/or code that cannot be parsed. Please report these issues at
https://github.com/magicmq/docs-translator/issues
"""
from java.util import EventObject
from org.bukkit.conversations import *
from typing import Any, Callable, Iterable, Tuple


class ConversationAbandonedEvent(EventObject):
    """
    ConversationAbandonedEvent contains information about an abandoned
    conversation.
    """

    def __init__(self, conversation: "Conversation"):
        ...


    def __init__(self, conversation: "Conversation", canceller: "ConversationCanceller"):
        ...


    def getCanceller(self) -> "ConversationCanceller":
        """
        Gets the object that caused the conversation to be abandoned.

        Returns
        - The object that abandoned the conversation.
        """
        ...


    def getContext(self) -> "ConversationContext":
        """
        Gets the abandoned conversation's conversation context.

        Returns
        - The abandoned conversation's conversation context.
        """
        ...


    def gracefulExit(self) -> bool:
        """
        Indicates how the conversation was abandoned - naturally as part of the
        prompt chain or prematurely via a ConversationCanceller.

        Returns
        - True if the conversation is abandoned gracefully by a Prompt returning null or the next prompt. False of the
            conversations is abandoned prematurely by a ConversationCanceller.
        """
        ...



"""
Python module generated from Java source file java.util.concurrent.LinkedBlockingDeque

Java source file obtained from artifact jdk version jdk

Because this Python module is automatically generated, it may contain errors
and/or code that cannot be parsed. Please report these issues at
https://github.com/magicmq/docs-translator/issues
"""
from java.util import AbstractQueue
from java.util import Iterator
from java.util import NoSuchElementException
from java.util import Objects
from java.util import Spliterator
from java.util.concurrent import *
from java.util.concurrent.locks import Condition
from java.util.concurrent.locks import ReentrantLock
from java.util.function import Consumer
from java.util.function import Predicate
from typing import Any, Callable, Iterable, Tuple


class LinkedBlockingDeque(AbstractQueue, BlockingDeque, Serializable):
    """
    An optionally-bounded BlockingDeque blocking deque based on
    linked nodes.
    
    The optional capacity bound constructor argument serves as a
    way to prevent excessive expansion. The capacity, if unspecified,
    is equal to Integer.MAX_VALUE.  Linked nodes are
    dynamically created upon each insertion unless this would bring the
    deque above capacity.
    
    Most operations run in constant time (ignoring time spent
    blocking).  Exceptions include .remove(Object) remove,
    .removeFirstOccurrence removeFirstOccurrence, .removeLastOccurrence removeLastOccurrence, .contains
    contains, .iterator iterator.remove(), and the bulk
    operations, all of which run in linear time.
    
    This class and its iterator implement all of the *optional*
    methods of the Collection and Iterator interfaces.
    
    This class is a member of the
    <a href="/java.base/java/util/package-summary.html#CollectionsFramework">
    Java Collections Framework</a>.
    
    Type `<E>`: the type of elements held in this deque

    Author(s)
    - Doug Lea

    Since
    - 1.6
    """

    def __init__(self):
        """
        Creates a `LinkedBlockingDeque` with a capacity of
        Integer.MAX_VALUE.
        """
        ...


    def __init__(self, capacity: int):
        """
        Creates a `LinkedBlockingDeque` with the given (fixed) capacity.

        Arguments
        - capacity: the capacity of this deque

        Raises
        - IllegalArgumentException: if `capacity` is less than 1
        """
        ...


    def __init__(self, c: Iterable["E"]):
        """
        Creates a `LinkedBlockingDeque` with a capacity of
        Integer.MAX_VALUE, initially containing the elements of
        the given collection, added in traversal order of the
        collection's iterator.

        Arguments
        - c: the collection of elements to initially contain

        Raises
        - NullPointerException: if the specified collection or any
                of its elements are null
        """
        ...


    def addFirst(self, e: "E") -> None:
        """
        Raises
        - IllegalStateException: if this deque is full
        - NullPointerException: 
        """
        ...


    def addLast(self, e: "E") -> None:
        """
        Raises
        - IllegalStateException: if this deque is full
        - NullPointerException: 
        """
        ...


    def offerFirst(self, e: "E") -> bool:
        """
        Raises
        - NullPointerException: 
        """
        ...


    def offerLast(self, e: "E") -> bool:
        """
        Raises
        - NullPointerException: 
        """
        ...


    def putFirst(self, e: "E") -> None:
        """
        Raises
        - NullPointerException: 
        - InterruptedException: 
        """
        ...


    def putLast(self, e: "E") -> None:
        """
        Raises
        - NullPointerException: 
        - InterruptedException: 
        """
        ...


    def offerFirst(self, e: "E", timeout: int, unit: "TimeUnit") -> bool:
        """
        Raises
        - NullPointerException: 
        - InterruptedException: 
        """
        ...


    def offerLast(self, e: "E", timeout: int, unit: "TimeUnit") -> bool:
        """
        Raises
        - NullPointerException: 
        - InterruptedException: 
        """
        ...


    def removeFirst(self) -> "E":
        """
        Raises
        - NoSuchElementException: 
        """
        ...


    def removeLast(self) -> "E":
        """
        Raises
        - NoSuchElementException: 
        """
        ...


    def pollFirst(self) -> "E":
        ...


    def pollLast(self) -> "E":
        ...


    def takeFirst(self) -> "E":
        ...


    def takeLast(self) -> "E":
        ...


    def pollFirst(self, timeout: int, unit: "TimeUnit") -> "E":
        ...


    def pollLast(self, timeout: int, unit: "TimeUnit") -> "E":
        ...


    def getFirst(self) -> "E":
        """
        Raises
        - NoSuchElementException: 
        """
        ...


    def getLast(self) -> "E":
        """
        Raises
        - NoSuchElementException: 
        """
        ...


    def peekFirst(self) -> "E":
        ...


    def peekLast(self) -> "E":
        ...


    def removeFirstOccurrence(self, o: "Object") -> bool:
        ...


    def removeLastOccurrence(self, o: "Object") -> bool:
        ...


    def add(self, e: "E") -> bool:
        """
        Inserts the specified element at the end of this deque unless it would
        violate capacity restrictions.  When using a capacity-restricted deque,
        it is generally preferable to use method .offer(Object) offer.
        
        This method is equivalent to .addLast.

        Raises
        - IllegalStateException: if this deque is full
        - NullPointerException: if the specified element is null
        """
        ...


    def offer(self, e: "E") -> bool:
        """
        Raises
        - NullPointerException: if the specified element is null
        """
        ...


    def put(self, e: "E") -> None:
        """
        Raises
        - NullPointerException: 
        - InterruptedException: 
        """
        ...


    def offer(self, e: "E", timeout: int, unit: "TimeUnit") -> bool:
        """
        Raises
        - NullPointerException: 
        - InterruptedException: 
        """
        ...


    def remove(self) -> "E":
        """
        Retrieves and removes the head of the queue represented by this deque.
        This method differs from .poll() poll() only in that it throws an
        exception if this deque is empty.
        
        This method is equivalent to .removeFirst() removeFirst.

        Returns
        - the head of the queue represented by this deque

        Raises
        - NoSuchElementException: if this deque is empty
        """
        ...


    def poll(self) -> "E":
        ...


    def take(self) -> "E":
        ...


    def poll(self, timeout: int, unit: "TimeUnit") -> "E":
        ...


    def element(self) -> "E":
        """
        Retrieves, but does not remove, the head of the queue represented by
        this deque.  This method differs from .peek() peek() only in that
        it throws an exception if this deque is empty.
        
        This method is equivalent to .getFirst() getFirst.

        Returns
        - the head of the queue represented by this deque

        Raises
        - NoSuchElementException: if this deque is empty
        """
        ...


    def peek(self) -> "E":
        ...


    def remainingCapacity(self) -> int:
        """
        Returns the number of additional elements that this deque can ideally
        (in the absence of memory or resource constraints) accept without
        blocking. This is always equal to the initial capacity of this deque
        less the current `size` of this deque.
        
        Note that you *cannot* always tell if an attempt to insert
        an element will succeed by inspecting `remainingCapacity`
        because it may be the case that another thread is about to
        insert or remove an element.
        """
        ...


    def drainTo(self, c: Iterable["E"]) -> int:
        """
        Raises
        - UnsupportedOperationException: 
        - ClassCastException: 
        - NullPointerException: 
        - IllegalArgumentException: 
        """
        ...


    def drainTo(self, c: Iterable["E"], maxElements: int) -> int:
        """
        Raises
        - UnsupportedOperationException: 
        - ClassCastException: 
        - NullPointerException: 
        - IllegalArgumentException: 
        """
        ...


    def push(self, e: "E") -> None:
        """
        Raises
        - IllegalStateException: if this deque is full
        - NullPointerException: 
        """
        ...


    def pop(self) -> "E":
        """
        Raises
        - NoSuchElementException: 
        """
        ...


    def remove(self, o: "Object") -> bool:
        """
        Removes the first occurrence of the specified element from this deque.
        If the deque does not contain the element, it is unchanged.
        More formally, removes the first element `e` such that
        `o.equals(e)` (if such an element exists).
        Returns `True` if this deque contained the specified element
        (or equivalently, if this deque changed as a result of the call).
        
        This method is equivalent to
        .removeFirstOccurrence(Object) removeFirstOccurrence.

        Arguments
        - o: element to be removed from this deque, if present

        Returns
        - `True` if this deque changed as a result of the call
        """
        ...


    def size(self) -> int:
        """
        Returns the number of elements in this deque.

        Returns
        - the number of elements in this deque
        """
        ...


    def contains(self, o: "Object") -> bool:
        """
        Returns `True` if this deque contains the specified element.
        More formally, returns `True` if and only if this deque contains
        at least one element `e` such that `o.equals(e)`.

        Arguments
        - o: object to be checked for containment in this deque

        Returns
        - `True` if this deque contains the specified element
        """
        ...


    def addAll(self, c: Iterable["E"]) -> bool:
        """
        Appends all of the elements in the specified collection to the end of
        this deque, in the order that they are returned by the specified
        collection's iterator.  Attempts to `addAll` of a deque to
        itself result in `IllegalArgumentException`.

        Arguments
        - c: the elements to be inserted into this deque

        Returns
        - `True` if this deque changed as a result of the call

        Raises
        - NullPointerException: if the specified collection or any
                of its elements are null
        - IllegalArgumentException: if the collection is this deque
        - IllegalStateException: if this deque is full

        See
        - .add(Object)
        """
        ...


    def toArray(self) -> list["Object"]:
        """
        Returns an array containing all of the elements in this deque, in
        proper sequence (from first to last element).
        
        The returned array will be "safe" in that no references to it are
        maintained by this deque.  (In other words, this method must allocate
        a new array).  The caller is thus free to modify the returned array.
        
        This method acts as bridge between array-based and collection-based
        APIs.

        Returns
        - an array containing all of the elements in this deque
        """
        ...


    def toArray(self, a: list["T"]) -> list["T"]:
        """
        Returns an array containing all of the elements in this deque, in
        proper sequence; the runtime type of the returned array is that of
        the specified array.  If the deque fits in the specified array, it
        is returned therein.  Otherwise, a new array is allocated with the
        runtime type of the specified array and the size of this deque.
        
        If this deque fits in the specified array with room to spare
        (i.e., the array has more elements than this deque), the element in
        the array immediately following the end of the deque is set to
        `null`.
        
        Like the .toArray() method, this method acts as bridge between
        array-based and collection-based APIs.  Further, this method allows
        precise control over the runtime type of the output array, and may,
        under certain circumstances, be used to save allocation costs.
        
        Suppose `x` is a deque known to contain only strings.
        The following code can be used to dump the deque into a newly
        allocated array of `String`:
        
        ``` `String[] y = x.toArray(new String[0]);````
        
        Note that `toArray(new Object[0])` is identical in function to
        `toArray()`.

        Arguments
        - a: the array into which the elements of the deque are to
                 be stored, if it is big enough; otherwise, a new array of the
                 same runtime type is allocated for this purpose

        Returns
        - an array containing all of the elements in this deque

        Raises
        - ArrayStoreException: if the runtime type of the specified array
                is not a supertype of the runtime type of every element in
                this deque
        - NullPointerException: if the specified array is null
        """
        ...


    def toString(self) -> str:
        ...


    def clear(self) -> None:
        """
        Atomically removes all of the elements from this deque.
        The deque will be empty after this call returns.
        """
        ...


    def iterator(self) -> Iterator["E"]:
        """
        Returns an iterator over the elements in this deque in proper sequence.
        The elements will be returned in order from first (head) to last (tail).
        
        The returned iterator is
        <a href="package-summary.html#Weakly">*weakly consistent*</a>.

        Returns
        - an iterator over the elements in this deque in proper sequence
        """
        ...


    def descendingIterator(self) -> Iterator["E"]:
        """
        Returns an iterator over the elements in this deque in reverse
        sequential order.  The elements will be returned in order from
        last (tail) to first (head).
        
        The returned iterator is
        <a href="package-summary.html#Weakly">*weakly consistent*</a>.

        Returns
        - an iterator over the elements in this deque in reverse order
        """
        ...


    def spliterator(self) -> "Spliterator"["E"]:
        """
        Returns a Spliterator over the elements in this deque.
        
        The returned spliterator is
        <a href="package-summary.html#Weakly">*weakly consistent*</a>.
        
        The `Spliterator` reports Spliterator.CONCURRENT,
        Spliterator.ORDERED, and Spliterator.NONNULL.

        Returns
        - a `Spliterator` over the elements in this deque

        Since
        - 1.8

        Unknown Tags
        - The `Spliterator` implements `trySplit` to permit limited
        parallelism.
        """
        ...


    def forEach(self, action: "Consumer"["E"]) -> None:
        """
        Raises
        - NullPointerException: 
        """
        ...


    def removeIf(self, filter: "Predicate"["E"]) -> bool:
        """
        Raises
        - NullPointerException: 
        """
        ...


    def removeAll(self, c: Iterable[Any]) -> bool:
        """
        Raises
        - NullPointerException: 
        """
        ...


    def retainAll(self, c: Iterable[Any]) -> bool:
        """
        Raises
        - NullPointerException: 
        """
        ...



"""
Python module generated from Java source file com.google.common.primitives.UnsignedLongs

Java source file obtained from artifact guava version 33.3.1-jre

Because this Python module is automatically generated, it may contain errors
and/or code that cannot be parsed. Please report these issues at
https://github.com/magicmq/docs-translator/issues
"""
from com.google.common.annotations import GwtCompatible
from com.google.common.primitives import *
from com.google.errorprone.annotations import CanIgnoreReturnValue
from java.math import BigInteger
from java.util import Arrays
from java.util import Comparator
from typing import Any, Callable, Iterable, Tuple


class UnsignedLongs:
    """
    Static utility methods pertaining to `long` primitives that interpret values as
    *unsigned* (that is, any negative value `x` is treated as the positive value `2^64 + x`). The methods for which signedness is not an issue are in Longs, as well as
    signed versions of methods for which signedness is an issue.
    
    In addition, this class provides several static methods for converting a `long` to a
    `String` and a `String` to a `long` that treat the `long` as an unsigned
    number.
    
    Users of these utilities must be *extremely careful* not to mix up signed and unsigned
    `long` values. When possible, it is recommended that the UnsignedLong wrapper class
    be used, at a small efficiency penalty, to enforce the distinction in the type system.
    
    See the Guava User Guide article on <a
    href="https://github.com/google/guava/wiki/PrimitivesExplained#unsigned-support">unsigned
    primitive utilities</a>.

    Author(s)
    - Colin Evans

    Since
    - 10.0
    """

    MAX_VALUE = -1L


    @staticmethod
    def compare(a: int, b: int) -> int:
        """
        Compares the two specified `long` values, treating them as unsigned values between `0` and `2^64 - 1` inclusive.
        
        **Note:** this method is now unnecessary and should be treated as deprecated; use the
        equivalent Long.compareUnsigned(long, long) method instead.

        Arguments
        - a: the first unsigned `long` to compare
        - b: the second unsigned `long` to compare

        Returns
        - a negative value if `a` is less than `b`; a positive value if `a` is
            greater than `b`; or zero if they are equal
        """
        ...


    @staticmethod
    def min(*array: Tuple[int, ...]) -> int:
        """
        Returns the least value present in `array`, treating values as unsigned.

        Arguments
        - array: a *nonempty* array of unsigned `long` values

        Returns
        - the value present in `array` that is less than or equal to every other value in
            the array according to .compare

        Raises
        - IllegalArgumentException: if `array` is empty
        """
        ...


    @staticmethod
    def max(*array: Tuple[int, ...]) -> int:
        """
        Returns the greatest value present in `array`, treating values as unsigned.

        Arguments
        - array: a *nonempty* array of unsigned `long` values

        Returns
        - the value present in `array` that is greater than or equal to every other value
            in the array according to .compare

        Raises
        - IllegalArgumentException: if `array` is empty
        """
        ...


    @staticmethod
    def join(separator: str, *array: Tuple[int, ...]) -> str:
        """
        Returns a string containing the supplied unsigned `long` values separated by `separator`. For example, `join("-", 1, 2, 3)` returns the string `"1-2-3"`.

        Arguments
        - separator: the text that should appear between consecutive values in the resulting string
            (but not at the start or end)
        - array: an array of unsigned `long` values, possibly empty
        """
        ...


    @staticmethod
    def lexicographicalComparator() -> "Comparator"[list[int]]:
        """
        Returns a comparator that compares two arrays of unsigned `long` values <a
        href="http://en.wikipedia.org/wiki/Lexicographical_order">lexicographically</a>. That is, it
        compares, using .compare(long, long)), the first pair of values that follow any common
        prefix, or when one array is a prefix of the other, treats the shorter array as the lesser. For
        example, `[] < [1L] < [1L, 2L] < [2L] < [1L << 63]`.
        
        The returned comparator is inconsistent with Object.equals(Object) (since arrays
        support only identity equality), but it is consistent with Arrays.equals(long[],
        long[]).
        """
        ...


    @staticmethod
    def sort(array: list[int]) -> None:
        """
        Sorts the array, treating its elements as unsigned 64-bit integers.

        Since
        - 23.1
        """
        ...


    @staticmethod
    def sort(array: list[int], fromIndex: int, toIndex: int) -> None:
        """
        Sorts the array between `fromIndex` inclusive and `toIndex` exclusive, treating its
        elements as unsigned 64-bit integers.

        Since
        - 23.1
        """
        ...


    @staticmethod
    def sortDescending(array: list[int]) -> None:
        """
        Sorts the elements of `array` in descending order, interpreting them as unsigned 64-bit
        integers.

        Since
        - 23.1
        """
        ...


    @staticmethod
    def sortDescending(array: list[int], fromIndex: int, toIndex: int) -> None:
        """
        Sorts the elements of `array` between `fromIndex` inclusive and `toIndex`
        exclusive in descending order, interpreting them as unsigned 64-bit integers.

        Since
        - 23.1
        """
        ...


    @staticmethod
    def divide(dividend: int, divisor: int) -> int:
        """
        Returns dividend / divisor, where the dividend and divisor are treated as unsigned 64-bit
        quantities.
        
        **Java 8+ users:** use Long.divideUnsigned(long, long) instead.

        Arguments
        - dividend: the dividend (numerator)
        - divisor: the divisor (denominator)

        Raises
        - ArithmeticException: if divisor is 0
        """
        ...


    @staticmethod
    def remainder(dividend: int, divisor: int) -> int:
        """
        Returns dividend % divisor, where the dividend and divisor are treated as unsigned 64-bit
        quantities.
        
        **Java 8+ users:** use Long.remainderUnsigned(long, long) instead.

        Arguments
        - dividend: the dividend (numerator)
        - divisor: the divisor (denominator)

        Raises
        - ArithmeticException: if divisor is 0

        Since
        - 11.0
        """
        ...


    @staticmethod
    def parseUnsignedLong(string: str) -> int:
        """
        Returns the unsigned `long` value represented by the given decimal string.
        
        **Java 8+ users:** use Long.parseUnsignedLong(String) instead.

        Raises
        - NumberFormatException: if the string does not contain a valid unsigned `long`
            value
        - NullPointerException: if `string` is null (in contrast to Long.parseLong(String))
        """
        ...


    @staticmethod
    def parseUnsignedLong(string: str, radix: int) -> int:
        """
        Returns the unsigned `long` value represented by a string with the given radix.
        
        **Java 8+ users:** use Long.parseUnsignedLong(String, int) instead.

        Arguments
        - string: the string containing the unsigned `long` representation to be parsed.
        - radix: the radix to use while parsing `string`

        Raises
        - NumberFormatException: if the string does not contain a valid unsigned `long` with
            the given radix, or if `radix` is not between Character.MIN_RADIX and Character.MAX_RADIX.
        - NullPointerException: if `string` is null (in contrast to Long.parseLong(String))
        """
        ...


    @staticmethod
    def decode(stringValue: str) -> int:
        """
        Returns the unsigned `long` value represented by the given string.
        
        Accepts a decimal, hexadecimal, or octal number given by specifying the following prefix:
        
        
          - `0x`*HexDigits*
          - `0X`*HexDigits*
          - `.`*HexDigits*
          - `0`*OctalDigits*

        Raises
        - NumberFormatException: if the string does not contain a valid unsigned `long`
            value

        Since
        - 13.0
        """
        ...


    @staticmethod
    def toString(x: int) -> str:
        """
        Returns a string representation of x, where x is treated as unsigned.
        
        **Java 8+ users:** use Long.toUnsignedString(long) instead.
        """
        ...


    @staticmethod
    def toString(x: int, radix: int) -> str:
        """
        Returns a string representation of `x` for the given radix, where `x` is treated as
        unsigned.
        
        **Java 8+ users:** use Long.toUnsignedString(long, int) instead.

        Arguments
        - x: the value to convert to a string.
        - radix: the radix to use while working with `x`

        Raises
        - IllegalArgumentException: if `radix` is not between Character.MIN_RADIX
            and Character.MAX_RADIX.
        """
        ...



"""
Python module generated from Java source file com.google.common.xml.XmlEscapers

Java source file obtained from artifact guava version 33.3.1-jre

Because this Python module is automatically generated, it may contain errors
and/or code that cannot be parsed. Please report these issues at
https://github.com/magicmq/docs-translator/issues
"""
from com.google.common.annotations import GwtCompatible
from com.google.common.escape import Escaper
from com.google.common.escape import Escapers
from com.google.common.xml import *
from typing import Any, Callable, Iterable, Tuple


class XmlEscapers:
    """
    `Escaper` instances suitable for strings to be included in XML attribute values and
    elements' text contents. When possible, avoid manual escaping by using templating systems and
    high-level APIs that provide autoescaping. For example, consider <a
    href="http://www.xom.nu/">XOM</a> or <a href="http://www.jdom.org/">JDOM</a>.
    
    **Note:** Currently the escapers provided by this class do not escape any characters
    outside the ASCII character range. Unlike HTML escaping the XML escapers will not escape
    non-ASCII characters to their numeric entity replacements. These XML escapers provide the minimal
    level of escaping to ensure that the output can be safely included in a Unicode XML document.
    
    For details on the behavior of the escapers in this class, see sections <a
    href="http://www.w3.org/TR/2008/REC-xml-20081126/#charsets">2.2</a> and <a
    href="http://www.w3.org/TR/2008/REC-xml-20081126/#syntax">2.4</a> of the XML specification.

    Author(s)
    - David Beaumont

    Since
    - 15.0
    """

    @staticmethod
    def xmlContentEscaper() -> "Escaper":
        """
        Returns an Escaper instance that escapes special characters in a string so it can
        safely be included in an XML document as element content. See section <a
        href="http://www.w3.org/TR/2008/REC-xml-20081126/#syntax">2.4</a> of the XML specification.
        
        **Note:** Double and single quotes are not escaped, so it is **not safe** to use this
        escaper to escape attribute values. Use .xmlContentEscaper if the output can appear in
        element content or .xmlAttributeEscaper in attribute values.
        
        This escaper substitutes `0xFFFD` for non-whitespace control characters and the
        character values `0xFFFE` and `0xFFFF` which are not permitted in XML. For more
        detail see section <a href="http://www.w3.org/TR/2008/REC-xml-20081126/#charsets">2.2</a> of
        the XML specification.
        
        This escaper does not escape non-ASCII characters to their numeric character references
        (NCR). Any non-ASCII characters appearing in the input will be preserved in the output.
        Specifically "\r" (carriage return) is preserved in the output, which may result in it being
        silently converted to "\n" when the XML is parsed.
        
        This escaper does not treat surrogate pairs specially and does not perform Unicode
        validation on its input.
        """
        ...


    @staticmethod
    def xmlAttributeEscaper() -> "Escaper":
        """
        Returns an Escaper instance that escapes special characters in a string so it can
        safely be included in XML document as an attribute value. See section <a
        href="http://www.w3.org/TR/2008/REC-xml-20081126/#AVNormalize">3.3.3</a> of the XML
        specification.
        
        This escaper substitutes `0xFFFD` for non-whitespace control characters and the
        character values `0xFFFE` and `0xFFFF` which are not permitted in XML. For more
        detail see section <a href="http://www.w3.org/TR/2008/REC-xml-20081126/#charsets">2.2</a> of
        the XML specification.
        
        This escaper does not escape non-ASCII characters to their numeric character references
        (NCR). However, horizontal tab `'\t'`, line feed `'\n'` and carriage return `'\r'` are escaped to a corresponding NCR `"&.x9;"`, `"&.xA;"`, and `"&.xD;"`
        respectively. Any other non-ASCII characters appearing in the input will be preserved in the
        output.
        
        This escaper does not treat surrogate pairs specially and does not perform Unicode
        validation on its input.
        """
        ...



"""
Python module generated from Java source file com.google.gson.internal.bind.DefaultDateTypeAdapter

Java source file obtained from artifact gson version 2.11.0

Because this Python module is automatically generated, it may contain errors
and/or code that cannot be parsed. Please report these issues at
https://github.com/magicmq/docs-translator/issues
"""
from com.google.gson import Gson
from com.google.gson import JsonSyntaxException
from com.google.gson import TypeAdapter
from com.google.gson import TypeAdapterFactory
from com.google.gson.internal import JavaVersion
from com.google.gson.internal import PreJava9DateFormatProvider
from com.google.gson.internal.bind import *
from com.google.gson.internal.bind.util import ISO8601Utils
from com.google.gson.reflect import TypeToken
from com.google.gson.stream import JsonReader
from com.google.gson.stream import JsonToken
from com.google.gson.stream import JsonWriter
from java.io import IOException
from java.text import DateFormat
from java.text import ParseException
from java.text import ParsePosition
from java.text import SimpleDateFormat
from java.util import Date
from java.util import Locale
from java.util import Objects
from java.util import TimeZone
from typing import Any, Callable, Iterable, Tuple


class DefaultDateTypeAdapter(TypeAdapter):
    """
    This type adapter supports subclasses of date by defining a DefaultDateTypeAdapter.DateType and then using its `createAdapterFactory` methods.
    
    **Important:** Instances of this class (or rather the SimpleDateFormat they use)
    capture the current default Locale and TimeZone when they are created. Therefore
    avoid storing factories obtained from DateType in `static` fields, since they only
    create a single adapter instance and its behavior would then depend on when Gson classes are
    loaded first, and which default `Locale` and `TimeZone` was used at that point.

    Author(s)
    - Joel Leitch
    """

    DEFAULT_STYLE_FACTORY = // Because SimpleDateFormat captures the default TimeZone when it was created, let the factory
    // always create DefaultDateTypeAdapter instances (which are then cached by the Gson
    // instances) instead of having a single static DefaultDateTypeAdapter instance
    // Otherwise the behavior would depend on when an application first loads Gson classes and
    // which default TimeZone is set at that point, which would be quite brittle
    TypeAdapterFactory() {
    
        // we use a runtime check to make sure the 'T's equal
        @SuppressWarnings("unchecked")
        @Override
        public <T> TypeAdapter<T> create(Gson gson, TypeToken<T> typeToken) {
            return typeToken.getRawType() == Date.class ? (TypeAdapter<T>) DefaultDateTypeAdapter<>(DateType.DATE, DateFormat.DEFAULT, DateFormat.DEFAULT) : null;
        }
    
        @Override
        public String toString() {
            return "DefaultDateTypeAdapter#DEFAULT_STYLE_FACTORY";
        }
    }
    """
    Factory for Date adapters which use DateFormat.DEFAULT as style.
    """


    def write(self, out: "JsonWriter", value: "Date") -> None:
        ...


    def read(self, in: "JsonReader") -> "T":
        ...


    def toString(self) -> str:
        ...


    class DateType:

        DATE = DateType<Date>(Date.class) {
        
            @Override
            protected Date deserialize(Date date) {
                return date;
            }
        }


        def createAdapterFactory(self, datePattern: str) -> "TypeAdapterFactory":
            ...


        def createAdapterFactory(self, dateStyle: int, timeStyle: int) -> "TypeAdapterFactory":
            ...



"""
Python module generated from Java source file com.google.common.escape.CharEscaperBuilder

Java source file obtained from artifact guava version 33.3.1-jre

Because this Python module is automatically generated, it may contain errors
and/or code that cannot be parsed. Please report these issues at
https://github.com/magicmq/docs-translator/issues
"""
from com.google.common.annotations import GwtCompatible
from com.google.common.escape import *
from com.google.errorprone.annotations import CanIgnoreReturnValue
from javax.annotation import CheckForNull
from org.checkerframework.checker.nullness.qual import Nullable
from typing import Any, Callable, Iterable, Tuple


class CharEscaperBuilder:
    """
    Simple helper class to build a "sparse" array of objects based on the indexes that were added to
    it. The array will be from 0 to the maximum index given. All non-set indexes will contain null
    (so it's not really a sparse array, just a pseudo sparse array). The builder can also return a
    CharEscaper based on the generated array.

    Author(s)
    - Sven Mawson

    Since
    - 15.0
    """

    def __init__(self):
        """
        Construct a new sparse array builder.
        """
        ...


    def addEscape(self, c: str, r: str) -> "CharEscaperBuilder":
        """
        Add a new mapping from an index to an object to the escaping.
        """
        ...


    def addEscapes(self, cs: list[str], r: str) -> "CharEscaperBuilder":
        """
        Add multiple mappings at once for a particular index.
        """
        ...


    def toArray(self) -> list[list[str]]:
        """
        Convert this builder into an array of char[]s where the maximum index is the value of the
        highest character that has been seen. The array will be sparse in the sense that any unseen
        index will default to null.

        Returns
        - a "sparse" array that holds the replacement mappings.
        """
        ...


    def toEscaper(self) -> "Escaper":
        """
        Convert this builder into a char escaper which is just a decorator around the underlying array
        of replacement char[]s.

        Returns
        - an escaper that escapes based on the underlying array.
        """
        ...



"""
Python module generated from Java source file java.util.ServiceConfigurationError

Java source file obtained from artifact jdk version jdk

Because this Python module is automatically generated, it may contain errors
and/or code that cannot be parsed. Please report these issues at
https://github.com/magicmq/docs-translator/issues
"""
from java.util import *
from typing import Any, Callable, Iterable, Tuple


class ServiceConfigurationError(Error):

    def __init__(self, msg: str):
        """
        Constructs a new instance with the specified message.

        Arguments
        - msg: The message, or `null` if there is no message
        """
        ...


    def __init__(self, msg: str, cause: "Throwable"):
        """
        Constructs a new instance with the specified message and cause.

        Arguments
        - msg: The message, or `null` if there is no message
        - cause: The cause, or `null` if the cause is nonexistent
                       or unknown
        """
        ...



"""
Python module generated from Java source file org.bukkit.map.MapCursorCollection

Java source file obtained from artifact spigot-api version 1.21.4-R0.1-20250303.102353-42

Because this Python module is automatically generated, it may contain errors
and/or code that cannot be parsed. Please report these issues at
https://github.com/magicmq/docs-translator/issues
"""
from org.bukkit.map import *
from typing import Any, Callable, Iterable, Tuple


class MapCursorCollection:
    """
    Represents all the map cursors on a MapCanvas. Like MapCanvas, a
    MapCursorCollection is linked to a specific MapRenderer.
    """

    def size(self) -> int:
        """
        Get the amount of cursors in this collection.

        Returns
        - The size of this collection.
        """
        ...


    def getCursor(self, index: int) -> "MapCursor":
        """
        Get a cursor from this collection.

        Arguments
        - index: The index of the cursor.

        Returns
        - The MapCursor.
        """
        ...


    def removeCursor(self, cursor: "MapCursor") -> bool:
        """
        Remove a cursor from the collection.

        Arguments
        - cursor: The MapCursor to remove.

        Returns
        - Whether the cursor was removed successfully.
        """
        ...


    def addCursor(self, cursor: "MapCursor") -> "MapCursor":
        """
        Add a cursor to the collection.

        Arguments
        - cursor: The MapCursor to add.

        Returns
        - The MapCursor that was passed.
        """
        ...


    def addCursor(self, x: int, y: int, direction: int) -> "MapCursor":
        """
        Add a cursor to the collection.

        Arguments
        - x: The x coordinate, from -128 to 127.
        - y: The y coordinate, from -128 to 127.
        - direction: The facing of the cursor, from 0 to 15.

        Returns
        - The newly added MapCursor.
        """
        ...


    def addCursor(self, x: int, y: int, direction: int, type: int) -> "MapCursor":
        """
        Add a cursor to the collection.

        Arguments
        - x: The x coordinate, from -128 to 127.
        - y: The y coordinate, from -128 to 127.
        - direction: The facing of the cursor, from 0 to 15.
        - type: The type (color/style) of the map cursor.

        Returns
        - The newly added MapCursor.

        Deprecated
        - Magic value
        """
        ...


    def addCursor(self, x: int, y: int, direction: int, type: int, visible: bool) -> "MapCursor":
        """
        Add a cursor to the collection.

        Arguments
        - x: The x coordinate, from -128 to 127.
        - y: The y coordinate, from -128 to 127.
        - direction: The facing of the cursor, from 0 to 15.
        - type: The type (color/style) of the map cursor.
        - visible: Whether the cursor is visible.

        Returns
        - The newly added MapCursor.

        Deprecated
        - Magic value
        """
        ...


    def addCursor(self, x: int, y: int, direction: int, type: int, visible: bool, caption: str) -> "MapCursor":
        """
        Add a cursor to the collection.

        Arguments
        - x: The x coordinate, from -128 to 127.
        - y: The y coordinate, from -128 to 127.
        - direction: The facing of the cursor, from 0 to 15.
        - type: The type (color/style) of the map cursor.
        - visible: Whether the cursor is visible.
        - caption: banner caption

        Returns
        - The newly added MapCursor.

        Deprecated
        - Magic value
        """
        ...



"""
Python module generated from Java source file org.bukkit.event.entity.VillagerReputationChangeEvent

Java source file obtained from artifact spigot-api version 1.21.4-R0.1-20250303.102353-42

Because this Python module is automatically generated, it may contain errors
and/or code that cannot be parsed. Please report these issues at
https://github.com/magicmq/docs-translator/issues
"""
from com.google.common.base import Preconditions
from java.util import UUID
from org.bukkit import Bukkit
from org.bukkit.entity import Entity
from org.bukkit.entity import Villager
from org.bukkit.event import Cancellable
from org.bukkit.event import HandlerList
from org.bukkit.event.entity import *
from typing import Any, Callable, Iterable, Tuple


class VillagerReputationChangeEvent(EntityEvent, Cancellable):
    """
    Called whenever an entity's reputation with a villager changes.
    """

    def __init__(self, villager: "Villager", targetUUID: "UUID", reason: "Villager.ReputationEvent", reputationType: "Villager.ReputationType", oldValue: int, newValue: int, maxValue: int):
        ...


    def getTargetUUID(self) -> "UUID":
        """
        Get UUID of the entity for whom the reputation with a villager changes.

        Returns
        - UUID of the entity for whom the reputation with a villager
                changes
        """
        ...


    def getTarget(self) -> "Entity":
        """
        Get the Entity for whom the reputation with a villager changes.

        Returns
        - the Entity for whom the reputation with a villager changes,
                or `null` if it isn't found
        """
        ...


    def getReason(self) -> "Villager.ReputationEvent":
        """
        Get the reason of this reputation change.

        Returns
        - the reason of this reputation change
        """
        ...


    def getReputationType(self) -> "Villager.ReputationType":
        """
        Get the type of changed reputation.

        Returns
        - the type of changed reputation
        """
        ...


    def getOldValue(self) -> int:
        """
        Get the reputation value before the change.

        Returns
        - the reputation value before the change
        """
        ...


    def getNewValue(self) -> int:
        """
        Get new reputation value after the change.

        Returns
        - the reputation value after the change
        """
        ...


    def setNewValue(self, newValue: int) -> None:
        """
        Set new reputation value for this event.
        
        If the final value is below the reputation discard threshold, gossip
        associated with this reputation type will be removed.
        
        The provided value must be between 0 and
        VillagerReputationChangeEvent.getMaxValue(), otherwise an
        IllegalArgumentException will be thrown. Each reputation type
        may have a different maximum value.

        Arguments
        - newValue: the reputation value after the change

        See
        - Villager.ReputationType.getMaxValue()
        """
        ...


    def getMaxValue(self) -> int:
        """
        Get maximum value for the reputation type affected by this event.

        Returns
        - the maximum value for the reputation type affected by this event

        See
        - Villager.ReputationType.getMaxValue()
        """
        ...


    def isCancelled(self) -> bool:
        ...


    def setCancelled(self, cancel: bool) -> None:
        ...


    def getEntity(self) -> "Villager":
        ...


    def getHandlers(self) -> "HandlerList":
        ...


    @staticmethod
    def getHandlerList() -> "HandlerList":
        ...



"""
Python module generated from Java source file com.google.common.collect.ImmutableMapEntrySet

Java source file obtained from artifact guava version 33.3.1-jre

Because this Python module is automatically generated, it may contain errors
and/or code that cannot be parsed. Please report these issues at
https://github.com/magicmq/docs-translator/issues
"""
from com.google.common.annotations import GwtCompatible
from com.google.common.annotations import GwtIncompatible
from com.google.common.annotations import J2ktIncompatible
from com.google.common.collect import *
from java.io import InvalidObjectException
from java.io import ObjectInputStream
from java.io import Serializable
from java.util import Spliterator
from java.util.function import Consumer
from javax.annotation import CheckForNull
from org.checkerframework.checker.nullness.qual import Nullable
from typing import Any, Callable, Iterable, Tuple


class ImmutableMapEntrySet(CachingAsList):
    """
    `entrySet()` implementation for ImmutableMap.

    Author(s)
    - Kevin Bourrillion
    """

    def size(self) -> int:
        ...


    def contains(self, object: "Object") -> bool:
        ...


    def hashCode(self) -> int:
        ...



"""
Python module generated from Java source file com.google.common.collect.RegularImmutableSortedSet

Java source file obtained from artifact guava version 33.3.1-jre

Because this Python module is automatically generated, it may contain errors
and/or code that cannot be parsed. Please report these issues at
https://github.com/magicmq/docs-translator/issues
"""
from com.google.common.annotations import GwtCompatible
from com.google.common.annotations import GwtIncompatible
from com.google.common.annotations import J2ktIncompatible
from com.google.common.collect import *
from java.util import Collections
from java.util import Comparator
from java.util import Iterator
from java.util import NoSuchElementException
from java.util import Spliterator
from java.util.function import Consumer
from javax.annotation import CheckForNull
from org.checkerframework.checker.nullness.qual import Nullable
from typing import Any, Callable, Iterable, Tuple


class RegularImmutableSortedSet(ImmutableSortedSet):
    """
    An immutable sorted set with one or more elements. TODO(jlevy): Consider separate class for a
    single-element sorted set.

    Author(s)
    - Louis Wasserman
    """

    def iterator(self) -> "UnmodifiableIterator"["E"]:
        ...


    def descendingIterator(self) -> "UnmodifiableIterator"["E"]:
        ...


    def spliterator(self) -> "Spliterator"["E"]:
        ...


    def forEach(self, action: "Consumer"["E"]) -> None:
        ...


    def size(self) -> int:
        ...


    def contains(self, o: "Object") -> bool:
        ...


    def containsAll(self, targets: Iterable[Any]) -> bool:
        ...


    def equals(self, object: "Object") -> bool:
        ...


    def first(self) -> "E":
        ...


    def last(self) -> "E":
        ...


    def lower(self, element: "E") -> "E":
        ...


    def floor(self, element: "E") -> "E":
        ...


    def ceiling(self, element: "E") -> "E":
        ...


    def higher(self, element: "E") -> "E":
        ...



"""
Python module generated from Java source file org.bukkit.conversations.ConversationAbandonedListener

Java source file obtained from artifact spigot-api version 1.21.4-R0.1-20250303.102353-42

Because this Python module is automatically generated, it may contain errors
and/or code that cannot be parsed. Please report these issues at
https://github.com/magicmq/docs-translator/issues
"""
from java.util import EventListener
from org.bukkit.conversations import *
from typing import Any, Callable, Iterable, Tuple


class ConversationAbandonedListener(EventListener):
    """

    """

    def conversationAbandoned(self, abandonedEvent: "ConversationAbandonedEvent") -> None:
        """
        Called whenever a Conversation is abandoned.

        Arguments
        - abandonedEvent: Contains details about the abandoned
            conversation.
        """
        ...



"""
Python module generated from Java source file com.google.common.collect.Tables

Java source file obtained from artifact guava version 33.3.1-jre

Because this Python module is automatically generated, it may contain errors
and/or code that cannot be parsed. Please report these issues at
https://github.com/magicmq/docs-translator/issues
"""
from com.google.common.annotations import GwtCompatible
from com.google.common.annotations import J2ktIncompatible
from com.google.common.base import Function
from com.google.common.base import Objects
from com.google.common.base import Supplier
from com.google.common.collect import *
from com.google.common.collect.Table import Cell
from java.io import Serializable
from java.util import Collections
from java.util import Iterator
from java.util import SortedMap
from java.util import SortedSet
from java.util import Spliterator
from java.util.function import BinaryOperator
from java.util.stream import Collector
from javax.annotation import CheckForNull
from org.checkerframework.checker.nullness.qual import Nullable
from typing import Any, Callable, Iterable, Tuple


class Tables:
    """
    Provides static methods that involve a `Table`.
    
    See the Guava User Guide article on <a href=
    "https://github.com/google/guava/wiki/CollectionUtilitiesExplained#tables">`Tables`</a>.

    Author(s)
    - Louis Wasserman

    Since
    - 7.0
    """

    @staticmethod
    def toTable(rowFunction: "java.util.function.Function"["T", "R"], columnFunction: "java.util.function.Function"["T", "C"], valueFunction: "java.util.function.Function"["T", "V"], tableSupplier: "java.util.function.Supplier"["I"]) -> "Collector"["T", Any, "I"]:
        """
        Returns a Collector that accumulates elements into a `Table` created using the
        specified supplier, whose cells are generated by applying the provided mapping functions to the
        input elements. Cells are inserted into the generated `Table` in encounter order.
        
        If multiple input elements map to the same row and column, an `IllegalStateException`
        is thrown when the collection operation is performed.
        
        To collect to an ImmutableTable, use ImmutableTable.toImmutableTable.

        Since
        - 21.0
        """
        ...


    @staticmethod
    def toTable(rowFunction: "java.util.function.Function"["T", "R"], columnFunction: "java.util.function.Function"["T", "C"], valueFunction: "java.util.function.Function"["T", "V"], mergeFunction: "BinaryOperator"["V"], tableSupplier: "java.util.function.Supplier"["I"]) -> "Collector"["T", Any, "I"]:
        """
        Returns a Collector that accumulates elements into a `Table` created using the
        specified supplier, whose cells are generated by applying the provided mapping functions to the
        input elements. Cells are inserted into the generated `Table` in encounter order.
        
        If multiple input elements map to the same row and column, the specified merging function is
        used to combine the values. Like java.util.stream.Collectors.toMap(java.util.function.Function, java.util.function.Function,
        BinaryOperator, java.util.function.Supplier), this Collector throws a `NullPointerException` on null values returned from `valueFunction`, and treats nulls
        returned from `mergeFunction` as removals of that row/column pair.

        Since
        - 21.0
        """
        ...


    @staticmethod
    def immutableCell(rowKey: "R", columnKey: "C", value: "V") -> "Cell"["R", "C", "V"]:
        """
        Returns an immutable cell with the specified row key, column key, and value.
        
        The returned cell is serializable.

        Arguments
        - rowKey: the row key to be associated with the returned cell
        - columnKey: the column key to be associated with the returned cell
        - value: the value to be associated with the returned cell
        """
        ...


    @staticmethod
    def transpose(table: "Table"["R", "C", "V"]) -> "Table"["C", "R", "V"]:
        """
        Creates a transposed view of a given table that flips its row and column keys. In other words,
        calling `get(columnKey, rowKey)` on the generated table always returns the same value as
        calling `get(rowKey, columnKey)` on the original table. Updating the original table
        changes the contents of the transposed table and vice versa.
        
        The returned table supports update operations as long as the input table supports the
        analogous operation with swapped rows and columns. For example, in a HashBasedTable
        instance, `rowKeySet().iterator()` supports `remove()` but `columnKeySet().iterator()` doesn't. With a transposed HashBasedTable, it's the other
        way around.
        """
        ...


    @staticmethod
    def newCustomTable(backingMap: dict["R", dict["C", "V"]], factory: "Supplier"[dict["C", "V"]]) -> "Table"["R", "C", "V"]:
        """
        Creates a table that uses the specified backing map and factory. It can generate a table based
        on arbitrary Map classes.
        
        The `factory`-generated and `backingMap` classes determine the table iteration
        order. However, the table's `row()` method returns instances of a different class than
        `factory.get()` does.
        
        Call this method only when the simpler factory methods in classes like HashBasedTable and TreeBasedTable won't suffice.
        
        The views returned by the `Table` methods Table.column, Table.columnKeySet, and Table.columnMap have iterators that don't support `remove()`. Otherwise, all optional operations are supported. Null row keys, columns keys, and
        values are not supported.
        
        Lookups by row key are often faster than lookups by column key, because the data is stored
        in a `Map<R, Map<C, V>>`. A method call like `column(columnKey).get(rowKey)` still
        runs quickly, since the row key is provided. However, `column(columnKey).size()` takes
        longer, since an iteration across all row keys occurs.
        
        Note that this implementation is not synchronized. If multiple threads access this table
        concurrently and one of the threads modifies the table, it must be synchronized externally.
        
        The table is serializable if `backingMap`, `factory`, the maps generated by
        `factory`, and the table contents are all serializable.
        
        Note: the table assumes complete ownership over of `backingMap` and the maps returned
        by `factory`. Those objects should not be manually updated and they should not use soft,
        weak, or phantom references.

        Arguments
        - backingMap: place to store the mapping from each row key to its corresponding column key
            / value map
        - factory: supplier of new, empty maps that will each hold all column key / value mappings
            for a given row key

        Raises
        - IllegalArgumentException: if `backingMap` is not empty

        Since
        - 10.0
        """
        ...


    @staticmethod
    def transformValues(fromTable: "Table"["R", "C", "V1"], function: "Function"["V1", "V2"]) -> "Table"["R", "C", "V2"]:
        """
        Returns a view of a table where each value is transformed by a function. All other properties
        of the table, such as iteration order, are left intact.
        
        Changes in the underlying table are reflected in this view. Conversely, this view supports
        removal operations, and these are reflected in the underlying table.
        
        It's acceptable for the underlying table to contain null keys, and even null values provided
        that the function is capable of accepting null input. The transformed table might contain null
        values, if the function sometimes gives a null result.
        
        The returned table is not thread-safe or serializable, even if the underlying table is.
        
        The function is applied lazily, invoked when needed. This is necessary for the returned
        table to be a view, but it means that the function will be applied many times for bulk
        operations like Table.containsValue and `Table.toString()`. For this to perform
        well, `function` should be fast. To avoid lazy evaluation when the returned table doesn't
        need to be a view, copy the returned table into a new table of your choosing.

        Since
        - 10.0
        """
        ...


    @staticmethod
    def unmodifiableTable(table: "Table"["R", "C", "V"]) -> "Table"["R", "C", "V"]:
        """
        Returns an unmodifiable view of the specified table. This method allows modules to provide
        users with "read-only" access to internal tables. Query operations on the returned table "read
        through" to the specified table, and attempts to modify the returned table, whether direct or
        via its collection views, result in an `UnsupportedOperationException`.
        
        The returned table will be serializable if the specified table is serializable.
        
        Consider using an ImmutableTable, which is guaranteed never to change.

        Since
        - 11.0
        """
        ...


    @staticmethod
    def unmodifiableRowSortedTable(table: "RowSortedTable"["R", "C", "V"]) -> "RowSortedTable"["R", "C", "V"]:
        """
        Returns an unmodifiable view of the specified row-sorted table. This method allows modules to
        provide users with "read-only" access to internal tables. Query operations on the returned
        table "read through" to the specified table, and attempts to modify the returned table, whether
        direct or via its collection views, result in an `UnsupportedOperationException`.
        
        The returned table will be serializable if the specified table is serializable.

        Arguments
        - table: the row-sorted table for which an unmodifiable view is to be returned

        Returns
        - an unmodifiable view of the specified table

        Since
        - 11.0
        """
        ...


    @staticmethod
    def synchronizedTable(table: "Table"["R", "C", "V"]) -> "Table"["R", "C", "V"]:
        """
        Returns a synchronized (thread-safe) table backed by the specified table. In order to guarantee
        serial access, it is critical that **all** access to the backing table is accomplished
        through the returned table.
        
        It is imperative that the user manually synchronize on the returned table when accessing any
        of its collection views:
        
        ````Table<R, C, V> table = Tables.synchronizedTable(HashBasedTable.<R, C, V>create());
        ...
        Map<C, V> row = table.row(rowKey);  // Needn't be in synchronized block
        ...
        synchronized (table) {  // Synchronizing on table, not row!
          Iterator<Entry<C, V>> i = row.entrySet().iterator(); // Must be in synchronized block
          while (i.hasNext()) {
            foo(i.next());`
        }
        }```
        
        Failure to follow this advice may result in non-deterministic behavior.
        
        The returned table will be serializable if the specified table is serializable.

        Arguments
        - table: the table to be wrapped in a synchronized view

        Returns
        - a synchronized view of the specified table

        Since
        - 22.0
        """
        ...



"""
Python module generated from Java source file com.google.common.escape.CharEscaper

Java source file obtained from artifact guava version 33.3.1-jre

Because this Python module is automatically generated, it may contain errors
and/or code that cannot be parsed. Please report these issues at
https://github.com/magicmq/docs-translator/issues
"""
from com.google.common.annotations import GwtCompatible
from com.google.common.escape import *
from javax.annotation import CheckForNull
from typing import Any, Callable, Iterable, Tuple


class CharEscaper(Escaper):
    """
    An object that converts literal text into a format safe for inclusion in a particular context
    (such as an XML document). Typically (but not always), the inverse process of "unescaping" the
    text is performed automatically by the relevant parser.
    
    For example, an XML escaper would convert the literal string `"Foo<Bar>"` into `"Foo&lt;Bar&gt;"` to prevent `"<Bar>"` from being confused with an XML tag. When the
    resulting XML document is parsed, the parser API will return this text as the original literal
    string `"Foo<Bar>"`.
    
    A `CharEscaper` instance is required to be stateless, and safe when used concurrently by
    multiple threads.
    
    Popular escapers are defined as constants in classes like com.google.common.html.HtmlEscapers and com.google.common.xml.XmlEscapers. To create
    your own escapers extend this class and implement the .escape(char) method.

    Author(s)
    - Sven Mawson

    Since
    - 15.0
    """

    def escape(self, string: str) -> str:
        """
        Returns the escaped form of a given literal string.

        Arguments
        - string: the literal string to be escaped

        Returns
        - the escaped form of `string`

        Raises
        - NullPointerException: if `string` is null
        """
        ...



"""
Python module generated from Java source file com.google.common.base.JdkPattern

Java source file obtained from artifact guava version 33.3.1-jre

Because this Python module is automatically generated, it may contain errors
and/or code that cannot be parsed. Please report these issues at
https://github.com/magicmq/docs-translator/issues
"""
from com.google.common.annotations import GwtIncompatible
from com.google.common.base import *
from java.io import Serializable
from java.util.regex import Matcher
from java.util.regex import Pattern
from typing import Any, Callable, Iterable, Tuple


class JdkPattern(CommonPattern, Serializable):
    """
    A regex pattern implementation which is backed by the Pattern.
    """

    def matcher(self, t: "CharSequence") -> "CommonMatcher":
        ...


    def pattern(self) -> str:
        ...


    def flags(self) -> int:
        ...


    def toString(self) -> str:
        ...



"""
Python module generated from Java source file org.bukkit.plugin.messaging.ChannelNotRegisteredException

Java source file obtained from artifact spigot-api version 1.21.4-R0.1-20250303.102353-42

Because this Python module is automatically generated, it may contain errors
and/or code that cannot be parsed. Please report these issues at
https://github.com/magicmq/docs-translator/issues
"""
from org.bukkit.plugin.messaging import *
from typing import Any, Callable, Iterable, Tuple


class ChannelNotRegisteredException(RuntimeException):
    """
    Thrown if a Plugin attempts to send a message on an unregistered channel.
    """

    def __init__(self):
        ...


    def __init__(self, channel: str):
        ...
